#!/usr/bin/env python3

import os, subprocess

APP_NAME = "x-tile"
LOCALE_DIR = os.getcwd()


for dirpath, dirnames, filenames in os.walk(LOCALE_DIR):
    for filename in filenames:
        if dirpath == LOCALE_DIR:
            nation, ext = os.path.splitext(filename)
            if ext != ".po": continue
            bash_string = "msgmerge -U %s.po %s.pot" % (nation, APP_NAME)
            subprocess.call(bash_string, shell=True)
subprocess.call("rm *.po~", shell=True)



#
#      support.py
#
#      Copyright 2009-2020
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <codifies@gmail.com>.
#
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
import os
import subprocess
import ctypes
import re
import globs
import cons

def get_desktop_width_n_height():
    """Get Desktop Width and Height"""
    get_property(b"_NET_WORKAREA", glob.root, glob.XA_CARDINAL)
    if bool(glob.ret_pointer): return glob.ret_pointer[2], glob.ret_pointer[3]
    print("The WM does not set _NET_WORKAREA properly, x-tile may not work properly")
    screen = Gdk.Screen.get_default()
    rect = screen.get_monitor_geometry(0)
    return rect.width, rect.height

def is_window_in_curr_viewport(desktop_width, desktop_height, win_id):
    """Check the Window to be in the Current Viewport assuming Compiz is Running"""
    x, y, width, height, root = get_geom(win_id)
    if x < 0 or x >= desktop_width or y < 0 or y >= desktop_height: return False
    else: return True

def is_compiz_running():
    """Return True if Compiz is running"""
    get_property(b"_NET_SUPPORTING_WM_CHECK", glob.root, glob.XA_WINDOW)
    if bool(glob.ret_pointer) == False:
        get_property(b"_NET_SUPPORTING_WM_CHECK", glob.root, glob.XA_CARDINAL)
        if bool(glob.ret_pointer) == False:
            print("no 1!")
            return False
        else:
            sup_window = glob.ret_pointer[0]
    else:
        sup_window = glob.ret_pointer[0]
    get_property(b"_NET_WM_NAME", sup_window, glob.str_atom)
    if bool(glob.ret_pointer) == False:
        get_property(b"_NET_WM_NAME", sup_window, glob.XA_STRING)
        if bool(glob.ret_pointer) == False:
            print("no 2!")
            return False
    buff = ""
    keep_going = True
    i = 0
    while keep_going:
        eightchars = glob.ret_pointer[i]
        for j in range(8):
            curr_char = ((eightchars >> 8*j) & 0xff)
            if curr_char == 0:
                keep_going = False
                break
            buff += "%c" % curr_char
        i += 1
    print("WM =", buff)
    if buff == "Compiz": return True
    return False

def is_candidate_compiz_desktop(win):
    """Is this a candidate Compiz Desktop window"""
    if not glob.is_compiz_running: return False
    x, y, w, h, r = get_geom(win)
    if w > glob.monitors_areas[0][2] \
    and h > glob.monitors_areas[0][3] \
    and not is_window_Hmax(win) \
    and not is_window_Vmax(win):
        return True
    return False

def get_geom(win):
    """
    Status XQueryTree(display, w, root_return, parent_return, children_return, nchildren_return)
       Display *display;
       Window w;
       Window *root_return;
       Window *parent_return;
       Window **children_return;
       unsigned int *nchildren_return;
    """
    root_return = ctypes.c_ulong()
    parent_return = ctypes.c_ulong()
    children_return = ctypes.c_ulong()
    nchildren_return = ctypes.c_uint()

    glob.x11.XQueryTree(glob.disp,win,ctypes.byref(root_return),ctypes.byref(parent_return),
       ctypes.byref(children_return),ctypes.byref(nchildren_return))

    # undecorated windows have root as parent...
    if (glob.root!=parent_return.value):
        win=parent_return

    xwa = globs.XWindowAttributes()
    glob.x11.XGetWindowAttributes(glob.disp, win,ctypes.byref(xwa))

    get_property(b"_NET_FRAME_EXTENTS",win,glob.XA_CARDINAL)
    if bool(glob.ret_pointer)==False:
        l,r,t,b = (0,0,0,0)
    else:
        l,r,t,b = glob.ret_pointer[0], glob.ret_pointer[1], glob.ret_pointer[2], glob.ret_pointer[3]

    return xwa.x-l,xwa.y-t,xwa.width+(l+r),xwa.height+(t+b),xwa.root

def get_property(prop_name, window, data_type):  #128*256 longs (128kb) should be good enough...
    """   gets an x property puts in global return variables
          property name, window, return data type atom   """
    prop_atom = glob.x11.XInternAtom(glob.disp, prop_name, False)
    glob.x11.XGetWindowProperty(glob.disp, window, prop_atom, 0, 128*256, False, data_type,
                           ctypes.byref(glob.ret_type), ctypes.byref(glob.ret_format), ctypes.byref(glob.num_items),
                           ctypes.byref(glob.bytes_after), ctypes.byref(glob.ret_pointer))

def get_icon(win):
    """   this returns a Gdk.pixbuf of the windows icon
          converts argb into rgba in the process   """
    get_property(b"_NET_WM_ICON", win, glob.XA_CARDINAL)
    if not glob.ret_pointer:
        return None
    w = glob.ret_pointer[0]
    h = glob.ret_pointer[1]
    #print w,h
    if w > 48 or h > 48:
        return None
    s = w*h
    buff = b""
    i = 0
    while i<s:
        argb = glob.ret_pointer[i+2]
        i += 1
        buff = buff + (b"%c" % ((argb >> 16) & 0xff))
        buff = buff + (b"%c" % ((argb >> 8) & 0xff))
        buff = buff + (b"%c" % (argb & 0xff))
        buff = buff + (b"%c" % ((argb >> 24) & 0xff))
    pxbuf = GdkPixbuf.Pixbuf.new_from_data(buff, GdkPixbuf.Colorspace.RGB, True, 8, w, h, w*4)
    return pxbuf

def get_min_size(win):
    """ returns the minimum window size and its resize increment """
    supplied_return = ctypes.c_long(0)
    glob.x11.XGetWMNormalHints(glob.disp, win, ctypes.byref(glob.size_hints_return),
                               ctypes.byref(supplied_return))

def client_msg(win, msg, d0, d1, d2, d3, d4):
    """   sends a client message event used to change window properties   """
    event = globs.XClientMessageEvent()
    event.type = 33 #ClientMessage
    event.serial = 0
    event.send = True
    event.msgtype = glob.x11.XInternAtom(glob.disp, msg, False)
    event.window = win
    event.format = 32
    event.data0 = d0
    event.data1 = d1
    event.data2 = d2
    event.data3 = d3
    event.data4 = d4
    # mask is SubstructureRedirectMask | SubstructureNotifyMask (20&19)
    if not glob.x11.XSendEvent(glob.disp,glob.root,False, 1<<20 | 1<<19,ctypes.byref(event)):
        print("can't send message ", msg) # DEBUG

def moveresize(win, x, y, w, h, dest_workspace):
    """   moves window to the users current desktop removes states like
          maximised and fullscreen activates the window,
          raises it then finally! moves and resizes it """
    glob.x11.XSync(glob.disp, False)
    get_property(b"_NET_FRAME_EXTENTS", win, glob.XA_CARDINAL)
    if bool(glob.ret_pointer) == False:
        l,r,t,b = (0, 0, 0, 0)
    else:
        l,r,t,b = glob.ret_pointer[0], glob.ret_pointer[1], glob.ret_pointer[2], glob.ret_pointer[3]
    if dest_workspace < 0:
        get_property(b"_NET_CURRENT_DESKTOP", glob.root, glob.XA_CARDINAL)
        dest_workspace = glob.ret_pointer[0]
    ###
    # move myself to desktop
    client_msg(glob.root, b"_NET_CURRENT_DESKTOP", dest_workspace, 0, 0, 0, 0)
    # move window to desktop
    client_msg(win, b"_NET_WM_DESKTOP", dest_workspace, 0, 0, 0, 0)
    ###
    client_msg(win, b"_NET_WM_STATE", 0, glob.fscreen_atom, 0, 0, 0)
    client_msg(win, b"_NET_WM_STATE", 0, glob.maxv_atom,glob.maxh_atom, 0, 0)
    client_msg(win, b"_NET_ACTIVE_WINDOW", 0, 0, 0, 0, 0)
    glob.x11.XMapRaised(glob.disp, win)
    glob.x11.XSync(glob.disp, False)
    glob.x11.XMoveResizeWindow(glob.disp, win, int(x), int(y), int(w-(l+r)), int(h-(t+b))) # only interior size needs deco subtracting not position
    glob.x11.XSync(glob.disp, False)

def maximize(win):
    """ moves a window to the current desktop and maximises it """
    get_property(b"_NET_CURRENT_DESKTOP",glob.root,glob.XA_CARDINAL)
    cdt = glob.ret_pointer[0]
    client_msg(win, b"_NET_WM_DESKTOP", cdt, 0, 0, 0, 0)
    client_msg(win, b"_NET_WM_STATE", 1, glob.maxv_atom, glob.maxh_atom, 0, 0) # 1 = add
    glob.x11.XSync(glob.disp, False)

def unmaximize(win):
    """ moves a window to the current desktop and 'unmaximises' it """
    get_property(b"_NET_CURRENT_DESKTOP", glob.root, glob.XA_CARDINAL)
    cdt = glob.ret_pointer[0]
    client_msg(win, b"_NET_WM_DESKTOP", cdt, 0, 0, 0, 0)
    client_msg(win, b"_NET_WM_STATE", 0, glob.maxv_atom, glob.maxh_atom, 0, 0) # 0 = remove
    glob.x11.XSync(glob.disp, False)

def desktop_size():
    """   gets the "work area" usually space between panels """
    get_property(b"_NET_WORKAREA", glob.root, glob.XA_CARDINAL)
    if glob.num_items.value:
        return glob.ret_pointer[0], glob.ret_pointer[1], glob.ret_pointer[2], glob.ret_pointer[3]
    return 0,0,0,0

def is_inside(p, pdim, a, adim):
    """Returns True if p <---> p+dim is between a <---> a+dim"""
    return ( (p+pdim/2 > a) and (p+pdim/2 < a+adim) )

def subtract_areas(white_area, black_area):
    """Returns the white_area without the black_area"""
    #print "white_area", white_area
    #print "black_area", black_area
    if not is_inside(black_area[0], black_area[2], white_area[0], white_area[2])\
    or not is_inside(black_area[1], black_area[3], white_area[1], white_area[3]):
        #print "not is_inside"
        return white_area
    # ignore the desktop strut
    if black_area[2] > white_area[2]/2 and black_area[3] > white_area[3]/2:
        #print "strut ignored"
        return white_area
    # we have to understand whether the panel is top, bottom, left or right
    if black_area[2] < black_area[3]:
        # width < height => this is a left or right panel
        if black_area[0] == white_area[0]:
            #print "left panel"
            white_area[0] += black_area[2]
            white_area[2] -= black_area[2]
        else:
            #print "right panel"
            white_area[2] -= black_area[2]
    else:
        # width > height => this is a top or bottom panel
        if black_area[1] == white_area[1]:
            #print "top panel"
            white_area[1] += black_area[3]
            white_area[3] -= black_area[3]
        else:
            #print "bottom panel"
            white_area[3] -= black_area[3]
    return white_area

def translate_coords(win, x, y):
    """
    Bool XTranslateCoordinates(display, src_w, dest_w, src_x, src_y, dest_x_return, 
                               dest_y_return, child_return)
      Display *display;
      Window src_w, dest_w;
      int src_x, src_y;
      int *dest_x_return, *dest_y_return;
      Window *child_return;
    """
    child_return = ctypes.c_ulong()
    dest_x_return = ctypes.c_int()
    dest_y_return = ctypes.c_int()
    glob.x11.XTranslateCoordinates(glob.disp, win, glob.root, x, y,
                                   ctypes.byref(dest_x_return),
                                   ctypes.byref(dest_y_return),
                                   ctypes.byref(child_return))
    return dest_x_return.value, dest_y_return.value

def enumerate_strut_windows(display, rootwindow):
    """Retrieve the Strut Windows (the panels)"""
    strut_windows = []
    rootr = ctypes.c_ulong()
    parent = ctypes.c_ulong()
    children = ctypes.pointer(ctypes.c_ulong())
    noOfChildren = ctypes.c_int()
    x_return = ctypes.c_int()
    y_return = ctypes.c_int()
    width_return = ctypes.c_int()
    height_return = ctypes.c_int()
    dummy = ctypes.c_int()
    get_property(b"_NET_WM_STRUT_PARTIAL", rootwindow, glob.XA_CARDINAL)
    if glob.num_items.value:
        glob.x11.XGetGeometry(display, rootwindow, ctypes.byref(dummy), ctypes.byref(x_return), ctypes.byref(y_return),
                              ctypes.byref(width_return), ctypes.byref(height_return), ctypes.byref(dummy), ctypes.byref(dummy) )
        struct_origin = translate_coords(rootwindow, x_return.value, y_return.value)
        strut_windows.append([struct_origin[0], struct_origin[1], width_return.value, height_return.value])
    status = glob.x11.XQueryTree(display, rootwindow, ctypes.byref(rootr), ctypes.byref(parent), ctypes.byref(children), ctypes.byref(noOfChildren) )
    if noOfChildren.value:
        for i in range(0, noOfChildren.value):
            ptr = ctypes.cast(children[i], ctypes.POINTER(ctypes.c_uint) )
            strut_windows.extend(enumerate_strut_windows(display, children[i]))
        glob.x11.XFree(children)
    return strut_windows

def get_root_screen_index():
    """Get Screen Index"""
    xwa = globs.XWindowAttributes()
    glob.x11.XGetWindowAttributes(glob.disp, glob.root,ctypes.byref(xwa))
    glob.x11.XScreenNumberOfScreen.argtypes = [ctypes.c_void_p]
    screen_index = glob.x11.XScreenNumberOfScreen(xwa.screen)
    return screen_index

def get_process_name(p):
    # determine executable
    # we were only using just this routine from psutils so I
    # "borrowed" ahem it....
    if p == 0: return "?"
    try:
        _exe = os.readlink("/proc/%s/exe" % p)
    except OSError:
        if not os.path.exists("/proc/%s/stat" % p): return "?"
        f = open("/proc/%s/stat" % p)
        try:
            _exe = f.read().split(' ')[1].replace('(', '').replace(')', '')
        finally:
            f.close()
    # determine name and path
    if os.path.isabs(_exe):
        path, name = os.path.split(_exe)
    else:
        path = ''
        name = _exe
    if name[0:6]=="python":
        f=open("/proc/%s/cmdline" % p)
        try:
            cmdp=f.read().split(chr(0))
        finally:
            f.close()
        name=cmdp[1]
    return name


# these could be replaced by one general purpose routine but make
# code eleswhere easier to read...

def is_window_hidden(win):
    """ is a window hidden ie "minimised" ? """
    get_property(b"_NET_WM_STATE", win, glob.XA_ATOM)
    for i in range(0, glob.num_items.value):
        if glob.ret_pointer[i] == glob.hidden_atom: return True
    return False

def is_window_sticky(win):
    """ is a window sticky ? """
    get_property(b"_NET_WM_STATE", win, glob.XA_ATOM)
    for i in range(0, glob.num_items.value):
        if glob.ret_pointer[i] == glob.sticky_atom: return True
    return False

def is_window_Vmax(win):
    """ is a window vertically maximised ? """
    get_property(b"_NET_WM_STATE", win, glob.XA_ATOM)
    for i in range(0, glob.num_items.value):
        if glob.ret_pointer[i] == glob.maxv_atom: return True
    return False

def is_window_Hmax(win):
    """ is a window horizontally maximised ? """
    get_property(b"_NET_WM_STATE", win, glob.XA_ATOM)
    for i in range(0, glob.num_items.value):
        if glob.ret_pointer[i] == glob.maxh_atom: return True
    return False

def get_undo_element_from_win_id(win_id):
    """From win_id to Undo Snap Element"""
    if is_window_Vmax(win_id) or is_window_Hmax(win_id): is_maximized = 1
    else: is_maximized = 0
    win_geom = get_geom(win_id)
    return [str(win_id),
            str(is_maximized),
            str(win_geom[0]),
            str(win_geom[1]),
            str(win_geom[2]),
            str(win_geom[3])]

def undo_snap_write(gconf_client, undo_snap_vec):
    """Write Undo Snap to Disk"""
    undo_snap_str = ""
    for element in undo_snap_vec:
        undo_snap_str += ",".join(element) + " "
    undo_snap_str = undo_snap_str[:-1]
    gconf_client.set_string(cons.GCONF_UNDO % glob.screen_index, undo_snap_str)

def dialog_info(message, parent=None):
    """The Info dialog"""
    dialog = Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
                               type=Gtk.MessageType.INFO,
                               buttons=Gtk.ButtonsType.OK,
                               message_format=message)
    if parent != None: dialog.set_transient_for(parent)
    dialog.set_title(_("Info"))
    dialog.run()
    dialog.destroy()

def dialog_warning(message, parent=None):
    """The Warning dialog"""
    dialog = Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
                               type=Gtk.MessageType.WARNING,
                               buttons=Gtk.ButtonsType.OK,
                               message_format=message)
    if parent != None: dialog.set_transient_for(parent)
    dialog.set_title(_("Warning"))
    dialog.run()
    dialog.destroy()



#
#      core.py
#
#      Copyright 2009-2020
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <codifies@gmail.com>.
#
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

import support
import cons

def tile_vertically(windows_list, monitors_areas, dws):
    """Tile the Checked Windows Vertically"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_vertically_on_area(windows_list[0],
                                monitors_areas[0][0], monitors_areas[0][1],
                                monitors_areas[0][2], monitors_areas[0][3],
                                dws)
    if number_of_windows2 >= 2:
        tile_vertically_on_area(windows_list[1],
                                monitors_areas[1][0], monitors_areas[1][1],
                                monitors_areas[1][2], monitors_areas[1][3],
                                dws)

def tile_horizontally(windows_list, monitors_areas, dws):
    """Tile the Checked Windows Horizontally"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_horizontally_on_area(windows_list[0],
                                  monitors_areas[0][0], monitors_areas[0][1],
                                  monitors_areas[0][2], monitors_areas[0][3],
                                  dws)
    if number_of_windows2 >= 2:
        tile_horizontally_on_area(windows_list[1],
                                  monitors_areas[1][0], monitors_areas[1][1],
                                  monitors_areas[1][2], monitors_areas[1][3],
                                  dws)

def tile_quad(windows_list, monitors_areas, dws):
    """Tile the Given Windows Quad"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_quad_on_area(windows_list[0],
                          monitors_areas[0][0], monitors_areas[0][1],
                          monitors_areas[0][2], monitors_areas[0][3],
                          dws)
    if number_of_windows2 >= 2:
        tile_quad_on_area(windows_list[1],
                          monitors_areas[1][0], monitors_areas[1][1],
                          monitors_areas[1][2], monitors_areas[1][3],
                          dws)

def tile_grid(rows, cols, windows_list, monitors_areas, dws):
    """Tile the Given Windows in a rows by cols grid"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_grid_on_area(rows, cols, windows_list[0],
                          monitors_areas[0][0], monitors_areas[0][1],
                          monitors_areas[0][2], monitors_areas[0][3],
                          dws)
    if number_of_windows2 >= 2:
        tile_grid_on_area(rows, cols, windows_list[1],
                          monitors_areas[1][0], monitors_areas[1][1],
                          monitors_areas[1][2], monitors_areas[1][3],
                          dws)

def tile_triangle_up(windows_list, monitors_areas, dws):
    """Tile 3 Windows in Triangle Up Scheme"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_triangle_up_on_area(windows_list[0],
                                 monitors_areas[0][0], monitors_areas[0][1],
                                 monitors_areas[0][2], monitors_areas[0][3],
                                 dws)
    if number_of_windows2 >= 2:
        tile_triangle_up_on_area(windows_list[1],
                                 monitors_areas[1][0], monitors_areas[1][1],
                                 monitors_areas[1][2], monitors_areas[1][3],
                                 dws)

def tile_triangle_down(windows_list, monitors_areas, dws):
    """Tile 3 Windows in Triangle Up Scheme"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_triangle_down_on_area(windows_list[0],
                                   monitors_areas[0][0], monitors_areas[0][1],
                                   monitors_areas[0][2], monitors_areas[0][3],
                                   dws)
    if number_of_windows2 >= 2:
        tile_triangle_down_on_area(windows_list[1],
                                   monitors_areas[1][0], monitors_areas[1][1],
                                   monitors_areas[1][2], monitors_areas[1][3],
                                   dws)

def tile_triangle_left(windows_list, monitors_areas, dws):
    """Tile 3 Windows in Triangle Up Scheme"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_triangle_left_on_area(windows_list[0],
                                   monitors_areas[0][0], monitors_areas[0][1],
                                   monitors_areas[0][2], monitors_areas[0][3],
                                   dws)
    if number_of_windows2 >= 2:
        tile_triangle_left_on_area(windows_list[1],
                                   monitors_areas[1][0], monitors_areas[1][1],
                                   monitors_areas[1][2], monitors_areas[1][3],
                                   dws)

def tile_triangle_right(windows_list, monitors_areas, dws):
    """Tile 3 Windows in Triangle Up Scheme"""
    number_of_windows = len(windows_list[0])
    number_of_windows2 = len(windows_list[1])
    if number_of_windows >= 2:
        tile_triangle_right_on_area(windows_list[0],
                                    monitors_areas[0][0], monitors_areas[0][1],
                                    monitors_areas[0][2], monitors_areas[0][3],
                                    dws)
    if number_of_windows2 >= 2:
        tile_triangle_right_on_area(windows_list[1],
                                    monitors_areas[1][0], monitors_areas[1][1],
                                    monitors_areas[1][2], monitors_areas[1][3],
                                    dws)

def tile_vertically_on_area(windows_list, x, y, w, h, dws):
    """Tile the Given Windows Vertically on the Given Area"""
    step = (h / len(windows_list))
    win_num = 0
    for checked_window in windows_list:
        support.moveresize(checked_window, x, y+win_num*step, w , step, dws)
        win_num += 1

def tile_horizontally_on_area(windows_list, x, y, w, h, dws):
    """Tile the Given Windows Horizontally on the Given Area"""
    step = (w / len(windows_list))
    win_num = 0
    for checked_window in windows_list:
        support.moveresize(checked_window, x + win_num*step, y, step, h, dws)
        win_num += 1

def tile_quad_on_area(windows_list, x, y, w, h, dws):
    """Tile the Given Windows Quad on the Given Area"""
    if len(windows_list) > 4: windows_list = windows_list[0:4]
    for index,checked_window in enumerate(windows_list):
        if index in [1, 3]: xo = w/2
        else: xo = 0
        if index > 1: yo = h/2
        else: yo = 0
        support.moveresize(checked_window, (x + xo), (y + yo), w/2 , h/2, dws)

def tile_grid_on_area(rows, cols, windows_list, x, y, w, h, dws):
    """Tile the Given Windows Grid on the Given Area"""
    if len(windows_list) > rows*cols: windows_list = windows_list[0:rows*cols]
    step_h = (h / rows)
    step_w = (w / cols)
    for index,checked_window in enumerate(windows_list):
        xo = step_w*(index%cols)
        yo = step_h*(index/cols)
        support.moveresize(checked_window, x+xo, y+yo, step_w , step_h, dws)

def tile_triangle_up_on_area(windows_list, x, y, w, h, dws):
    """Tile 3 Windows in Triangle Up Scheme on the Given Area"""
    if len(windows_list) > 3: windows_list = windows_list[0:3]
    for index,checked_window in enumerate(windows_list):
        if index == 2: xo = w/2
        else: xo = 0
        if index > 0: yo = h/2
        else: yo = 0
        if index == 0: width = w
        else: width = w/2
        support.moveresize(checked_window, (x + xo), (y + yo), width, h/2, dws)

def tile_triangle_down_on_area(windows_list, x, y, w, h, dws):
    """Tile 3 Windows in Triangle Down Scheme on the Given Area"""
    if len(windows_list) > 3: windows_list = windows_list[0:3]
    for index,checked_window in enumerate(windows_list):
        if index == 1: xo = w/2
        else: xo = 0
        if index == 2: yo = h/2
        else: yo = 0
        if index == 2: width = w
        else: width = w/2
        support.moveresize(checked_window, (x + xo), (y + yo), width, h/2, dws)

def tile_triangle_left_on_area(windows_list, x, y, w, h, dws):
    """Tile 3 Windows in Triangle Left Scheme on the Given Area"""
    if len(windows_list) > 3: windows_list = windows_list[0:3]
    for index,checked_window in enumerate(windows_list):
        if index > 0: xo = w/2
        else: xo = 0
        if index == 2: yo = h/2
        else: yo = 0
        if index == 0: height = h
        else: height = h/2
        support.moveresize(checked_window, (x + xo), (y + yo), w/2 , height, dws)

def tile_triangle_right_on_area(windows_list, x, y, w, h, dws):
    """Tile 3 Windows in Triangle Right Scheme on the Given Area"""
    if len(windows_list) > 3: windows_list = windows_list[0:3]
    for index,checked_window in enumerate(windows_list):
        if index == 1: xo = w/2
        else: xo = 0
        if index == 2: yo = h/2
        else: yo = 0
        if index == 1: height = h
        else: height = h/2
        support.moveresize(checked_window, (x + xo), (y + yo), w/2 , height, dws)



#
#      globs.py
#
#      Copyright 2009-2020
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <codifies@gmail.com>.
#
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

from gi.repository import Gtk
from gi.repository import Gdk
import ctypes
import ctypes.util
import cons
import support
import core

class XSizeHints(ctypes.Structure):
    """ XSizeHints structure (xlib) """
    _fields_ = [("flags",              ctypes.c_long),
                ("x",                  ctypes.c_int),  # Obsolete
                ("y",                  ctypes.c_int),  # Obsolete
                ("width",              ctypes.c_int),  # Obsolete
                ("height",             ctypes.c_int),  # Obsolete
                ("min_width",          ctypes.c_int),
                ("min_height",         ctypes.c_int),
                ("max_width",          ctypes.c_int),
                ("max_height",         ctypes.c_int),
                ("width_inc",          ctypes.c_int),
                ("height_inc",         ctypes.c_int),
                ("min_asp_width",      ctypes.c_int),
                ("min_asp_height",     ctypes.c_int),
                ("max_asp_width",      ctypes.c_int),
                ("max_asp_height",     ctypes.c_int),
                ("base_width",         ctypes.c_int),
                ("base_height",        ctypes.c_int),
                ("win_gravity",        ctypes.c_int)]

class XWindowAttributes(ctypes.Structure):
    """ XWindowAttributes structure (xlib) """
    _fields_ = [("x",                      ctypes.c_int32),
                ("y",                      ctypes.c_int32),
                ("width",                  ctypes.c_int32),
                ("height",                 ctypes.c_int32),
                ("border_width",           ctypes.c_int32),
                ("depth",                  ctypes.c_int32),
                ("visual",                 ctypes.c_ulong),
                ("root",                   ctypes.c_ulong),
                ("class",                  ctypes.c_int32),
                ("bit_gravity",            ctypes.c_int32),
                ("win_gravity",            ctypes.c_int32),
                ("backing_store",          ctypes.c_int32),
                ("backing_planes",         ctypes.c_ulong),
                ("backing_pixel",          ctypes.c_ulong),
                ("save_under",             ctypes.c_int32),
                ("colourmap",              ctypes.c_ulong),
                ("mapinstalled",           ctypes.c_uint32),
                ("map_state",              ctypes.c_uint32),
                ("all_event_masks",        ctypes.c_ulong),
                ("your_event_mask",        ctypes.c_ulong),
                ("do_not_propagate_mask",  ctypes.c_ulong),
                ("override_redirect",      ctypes.c_int32),
                ("screen",                 ctypes.c_ulong)]

class XClientMessageEvent(ctypes.Structure):
    """ XClientMessageEvent structure (xlib) """
    _fields_ = [("type",       ctypes.c_int),
                ("serial",     ctypes.c_long),
                ("send",       ctypes.c_byte),
                ("display",    ctypes.c_long),
                ("window",     ctypes.c_long),
                ("msgtype",    ctypes.c_long),
                ("format",     ctypes.c_int),
                ("data0",      ctypes.c_long),
                ("data1",      ctypes.c_long),
                ("data2",      ctypes.c_long),
                ("data3",      ctypes.c_long),
                ("data4",      ctypes.c_long)]


class GlobalsObject(object):
    """Global Variables"""

    def __init__(self):
        """Instantiate global vars"""
        support.glob = self
        # x11 reference to xlib library display and root window globals
        self.x11 = ctypes.CDLL(ctypes.util.find_library("X11"))
        self.x11.XOpenDisplay.restype = ctypes.c_void_p
        self.disp = ctypes.c_void_p(self.x11.XOpenDisplay(0))
        self.root = self.x11.XDefaultRootWindow(self.disp)
        # property atoms for moveresize
        # assigned once here so they are not recreated
        # every time moveresize is called
        self.fscreen_atom = self.x11.XInternAtom(self.disp, b"_NET_WM_STATE_FULLSCREEN", False)
        self.maxv_atom = self.x11.XInternAtom(self.disp, b"_NET_WM_STATE_MAXIMIZED_VERT", False)
        self.maxh_atom = self.x11.XInternAtom(self.disp, b"_NET_WM_STATE_MAXIMIZED_HORZ", False)
        self.hidden_atom = self.x11.XInternAtom(self.disp, b"_NET_WM_STATE_HIDDEN", False)
        self.sticky_atom = self.x11.XInternAtom(self.disp, b"_NET_WM_STATE_STICKY", False)
        self.str_atom = self.x11.XInternAtom(self.disp, b"UTF8_STRING", False)
        # GLOBAL returns for getwindowproperty
        self.ret_type = ctypes.c_long()
        self.ret_format = ctypes.c_long()
        self.num_items = ctypes.c_long()
        self.bytes_after = ctypes.c_long()
        self.ret_pointer = ctypes.pointer(ctypes.c_long())
        # xlib global "defines" for some standard atoms
        self.XA_CARDINAL = 6
        self.XA_WINDOW = 33
        self.XA_STRING = 31
        self.XA_ATOM = 4
        # GLOBAL size hints return
        self.size_hints_return = XSizeHints()
        self.screen_index = support.get_root_screen_index()
        self.str2_atom = self.x11.XInternAtom(self.disp, b"STRING", False)
        self.num_monitors = Gdk.Screen.get_default().get_n_monitors()
        self.is_compiz_running = support.is_compiz_running()
        self.desktop_width, self.desktop_height = support.get_desktop_width_n_height()

    def read_monitors_areas(self):
        """Read Monitor(s) Area(s)"""
        strut_windows = support.enumerate_strut_windows(self.disp, self.root)
        #print strut_windows
        screen = Gdk.Screen.get_default()
        self.num_monitors = screen.get_n_monitors()
        self.monitors_areas = []
        drawing_area_size = [0, 0]
        for num_monitor in range(self.num_monitors):
            rect = screen.get_monitor_geometry(num_monitor)
            self.monitors_areas.append([rect.x, rect.y, rect.width, rect.height])
            if rect.x + rect.width > drawing_area_size[0]: drawing_area_size[0] = rect.x + rect.width
            if rect.y + rect.height > drawing_area_size[1]: drawing_area_size[1] = rect.y + rect.height
            for strut_win in strut_windows:
                self.monitors_areas[-1] = support.subtract_areas(self.monitors_areas[-1], strut_win)
        #print self.monitors_areas
        self.drawing_rect = (0, 0, drawing_area_size[0]/cons.DRAW_SCALE, drawing_area_size[1]/cons.DRAW_SCALE)



#!/usr/bin/env python3

import os, subprocess

APP_NAME = "x-tile"
LOCALE_DIR = os.getcwd()


for dirpath, dirnames, filenames in os.walk(LOCALE_DIR):
    for filename in filenames:
        if dirpath == LOCALE_DIR:
            nation, ext = os.path.splitext(filename)
            if ext != ".po": continue
            nation_dir = os.path.join(LOCALE_DIR, nation)
            if not os.path.isdir(nation_dir): os.mkdir(nation_dir)
            messages_dir = os.path.join(nation_dir, "LC_MESSAGES")
            if not os.path.isdir(messages_dir): os.mkdir(messages_dir)
            bash_string = "msgfmt --output-file=%s/LC_MESSAGES/%s.mo %s.po" % (nation, APP_NAME, nation)
            subprocess.call(bash_string, shell=True)



#
#      cons.py
#
#      Copyright 2009-2022
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <codifies@gmail.com>.
#
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

import os

APP_NAME = "x-tile"
VERSION = "3.3"
if os.path.isdir("glade"):
    GLADE_PATH = os.path.join(os.getcwd(), "glade/")
    LOCALE_PATH = "locale/"
else:
    GLADE_PATH = "/usr/share/x-tile/glade/"
    LOCALE_PATH = "/usr/share/locale/"
CONFIG_DIR = os.path.join(os.path.expanduser('~'), '.config', APP_NAME)

AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es_AR', 'fr', 'it', 'pl', 'ru', 'zh_CN', 'zh_TW']

CMD_LINE_ACTIONS = ["z", "i", "y", "v", "h", "u", "d", "l", "r", "1", "2", "q", "g", "m", "M", "c"]

OVERRIDE_1 = 0
OVERRIDE_2 = 0
X1 = 0
Y1 = 0
W1 = 10
H1 = 10
X2 = 0
Y2 = 0
W2 = 10
H2 = 10
GRID_ROWS = 2
GRID_COLS = 2

WINNAMES_BLACKLIST = [b"x-nautilus-desktop", b"pcmanfm"]
PROCESSES_BLACKLIST = [b"lxpanel"]
#PROCESSES_PARENT_BLACKLIST = ["gnome-session"]

STR_TRUE = "True"
STR_FALSE = "False"

DRAW_SCALE = 4
DRAW_RGBS = (
               (1, 0, 0), # red
               (0, 1, 0), # green
               (0, 0, 1), # blue
               (1, 1, 0), # yellow
               (0, 1, 1), # cyan
               (1, 0, 1), # red violet
               (0.5, 0.5, 0.5), # grey
               (1, 1, 1), # white
               #(0, 0, 0), # black
               (1, 0.65, 0), # orange yellow
               (0.25, 1, 0), # yellow green
               (1, 0, 0.25), # red wine
             )

ICONS_FILENAMES = ['tile-vertically.svg',
                   'tile-horizontally.svg',
                   'tile-quad.svg',
                   'tile-grid.svg',
                   'tile-custom-1-set.svg',
                   'tile-custom-1-exe.svg',
                   'tile-custom-2-set.svg',
                   'tile-custom-2-exe.svg',
                   'tile-triangle-up.svg',
                   'tile-triangle-down.svg',
                   'tile-triangle-left.svg',
                   'tile-triangle-right.svg',
                   'reload-windows-list.svg',
                   'maximize-checked-windows.svg',
                   'unmaximize-checked-windows.svg',
                   'close-checked-windows.svg',
                   'invert-tiling.svg',
                   'cycle-tiling.svg',
                   'toolbar.png',
                   'quit-app.svg',
                   'help-contents.svg',
                   'checkbox_checked.svg',
                   'checkbox_unchecked.svg']

GCONF_DIR = "/apps/x-tile"
GCONF_LANG = "/apps/x-tile/language"
GCONF_UNDO = "/apps/x-tile/%s/undo"
GCONF_LATEST_TILING = "/apps/x-tile/%s/latest_tiling"
GCONF_CUSTOM_1 = "/apps/x-tile/%s/custom"
GCONF_CUSTOM_2 = "/apps/x-tile/%s/custom_2"
GCONF_EXIT_AFTER_TILE = "/apps/x-tile/%s/exit_after_tile"
GCONF_ONLY_CURR_DESK = "/apps/x-tile/%s/only_curr_desk"
GCONF_NOT_MINIMIZED = "/apps/x-tile/%s/not_minimized"
GCONF_TOOLBAR_ICON_SIZE = "/apps/x-tile/%s/toolbar_icon_size"
GCONF_PROCESS_BLACKLIST = "/apps/x-tile/%s/process_blacklist"
GCONF_PROCESS_WHITELIST = "/apps/x-tile/%s/process_whitelist"
GCONF_WIN_SIZE_W = "/apps/x-tile/%s/win_size_w"
GCONF_WIN_SIZE_H = "/apps/x-tile/%s/win_size_h"
GCONF_WIN_POSITION_X = "/apps/x-tile/%s/win_position_x"
GCONF_WIN_POSITION_Y = "/apps/x-tile/%s/win_position_y"
GCONF_SHOW_TOOLBAR = "/apps/x-tile/%s/show_toolbar"
GCONF_X1 = "/apps/x-tile/%s/x_1"
GCONF_Y1 = "/apps/x-tile/%s/y_1"
GCONF_W1 = "/apps/x-tile/%s/w_1"
GCONF_H1 = "/apps/x-tile/%s/h_1"
GCONF_X2 = "/apps/x-tile/%s/x_2"
GCONF_Y2 = "/apps/x-tile/%s/y_2"
GCONF_W2 = "/apps/x-tile/%s/w_2"
GCONF_H2 = "/apps/x-tile/%s/h_2"
GCONF_OVERRIDE_1 = "/apps/x-tile/%s/override_1"
GCONF_OVERRIDE_2 = "/apps/x-tile/%s/override_2"
GCONF_GRID_ROWS = "/apps/x-tile/%s/grid_rows"
GCONF_GRID_COLS = "/apps/x-tile/%s/grid_cols"
GCONF_SYSTRAY_ENABLE = "/apps/x-tile/%s/systray_enable"
GCONF_USE_APPIND = "/apps/x-tile/%s/use_appind"
GCONF_SYSTRAY_START = "/apps/x-tile/%s/systray_start"

UI_INFO = """
<ui>
    <menubar name='MenuBar'>
        <menu action='FileMenu'>
            <menuitem action='Filter'/>
            <menuitem action='DefaultSel'/>
            <menuitem action='Reload'/>
            <separator/>
            <menuitem action='QuitApp'/>
            <menuitem action='ExitApp'/>
        </menu>

        <menu action='EditMenu'>
            <menuitem action='Preferences'/>
            <separator/>
            <menuitem action='SelectAll'/>
            <menuitem action='DeselectAll'/>
            <separator/>
        </menu>

        <menu action='RowMenu'>
            <menuitem action='RowUp'/>
            <menuitem action='RowDown'/>
            <separator/>
            <menuitem action='RowDelete'/>
        </menu>

        <menu action='TileMenu'>
            <menuitem action='UnTile'/>
            <menuitem action='InverTile'/>
            <menuitem action='CycleTile'/>
            <separator/>
            <menuitem action='Vertically'/>
            <menuitem action='Horizontally'/>
            <menuitem action='Grid'/>
            <separator/>
            <menuitem action='Custom1Set'/>
            <menuitem action='Custom2Set'/>
            <menuitem action='Custom1Exe'/>
            <menuitem action='Custom2Exe'/>
            <separator/>
            <menuitem action='TriangleLeft'/>
            <menuitem action='TriangleRight'/>
            <menuitem action='TriangleUp'/>
            <menuitem action='TriangleDown'/>
            <separator/>
            <menuitem action='Quad'/>
            <separator/>
            <menuitem action='Maximize'/>
            <menuitem action='Unmaximize'/>
            <menuitem action='Close'/>
        </menu>

        <menu action='ViewMenu'>
            <menuitem action='ShowHideToolbar'/>
            <separator/>
            <menuitem action='IncreaseToolbarIconsSize'/>
            <menuitem action='DecreaseToolbarIconsSize'/>
        </menu>

        <menu action='HelpMenu'>
            <menuitem action='Help'/>
            <separator/>
            <menuitem action='About'/>
        </menu>
    </menubar>

    <toolbar name='ToolBar'>
        <toolitem action='SelectAll'/>
        <toolitem action='Reload'/>
        <separator/>
        <toolitem action='UnTile'/>
        <toolitem action='InverTile'/>
        <toolitem action='CycleTile'/>
        <separator/>
        <toolitem action='Vertically'/>
        <toolitem action='Horizontally'/>
        <toolitem action='Grid'/>
        <toolitem action='Custom1Exe'/>
        <toolitem action='Custom2Exe'/>
        <separator/>
        <toolitem action='Maximize'/>
        <toolitem action='Unmaximize'/>
        <toolitem action='Close'/>
    </toolbar>
    
    <popup name='SysTrayMenu'>
        <menuitem action='ShowHideMainWin'/>
        <separator/>
        <menuitem action='ExitApp'/>
        <separator/>
        <menuitem action='All_C'/>
        <menuitem action='All_U'/>
        <menuitem action='All_M'/>
        <separator/>
        <menuitem action='All_4'/>
        <separator/>
        <menuitem action='All_TD'/>
        <menuitem action='All_TU'/>
        <menuitem action='All_TR'/>
        <menuitem action='All_TL'/>
        <separator/>
        <menuitem action='All_C2'/>
        <menuitem action='All_C1'/>
        <separator/>
        <menuitem action='All_G'/>
        <menuitem action='All_H'/>
        <menuitem action='All_V'/>
        <separator/>
        <menuitem action='All_Cycle'/>
        <menuitem action='All_Invert'/>
        <menuitem action='All_Undo'/>
        <separator/>
        <menuitem action='All_About'/>
    </popup>
    
    <popup name='ListMenu'>
        <menuitem action='RowUp'/>
        <menuitem action='RowDown'/>
        <separator/>
        <menuitem action='RowDelete'/>
    </popup>
</ui>
"""

def get_entries(inst):
    """Returns the Menu Entries Given the Class Instance"""
    return [
    # name, stock id, label
    ( "FileMenu", None, _("_File") ),
    ( "EditMenu", None, _("_Edit") ),
    ( "RowMenu", None, _("_Row") ),
    ( "TileMenu", None, _("_Tile") ),
    ( "ViewMenu", None, _("_View") ),
    ( "HelpMenu", None, _("_Help") ),
    # name, stock id, label, accelerator, tooltip, callback
    ( "Filter", "gtk-properties", _("_Filter"), "<control>F", _("Filter Rows"), inst.dialog_filter),
    ( "DefaultSel", "gtk-properties", _("Selected by _Default"), "<control>D", _("Rows to be Selected by Default"), inst.dialog_selected_by_default),
    ( "Reload", "reload-windows-list", _("_Reload"), "F5", _("Reload the Windows List"), inst.reload_windows_list),
    ( "QuitApp", "quit-app", _("_Quit"), "<control>Q", _("Quit the Application"), inst.quit_application),
    ( "ExitApp", "quit-app", _("_Exit X Tile"), "<control><shift>Q", _("Exit from X Tile"), inst.quit_application_totally),
    ( "ShowHideMainWin", "tile-quad", _("Show/Hide _X Tile"), None, _("Toggle Show/Hide X Tile"), inst.toggle_show_hide_main_window),
    ( "Preferences", "gtk-preferences", _("_Preferences"), "<control><alt>P", _("Open the Preferences Window"), inst.dialog_preferences),
    ( "SelectAll", "checkbox_checked", _("Select _All"), "<control>A", _("Select All the Windows in the List"), inst.flag_all_rows),
    ( "DeselectAll", "checkbox_unchecked", _("Deselect A_ll"), "<control><shift>A", _("Deselect All the Windows in the List"), inst.unflag_all_rows),
    ( "Vertically", "tile-vertically", _("Tile _Vertically"), "<control>V", _("Tile Vertically The Checked Windows"), inst.tile_vertically),
    ( "Horizontally", "tile-horizontally", _("Tile _Horizontally"), "<control>H", _("Tile Horizontally The Checked Windows"), inst.tile_horizontally),
    ( "TriangleUp", "tile-triangle-up", _("_Triangle Up"), "<control>Up", _("Tile Triangle Up The Checked Windows"), inst.tile_triangle_up),
    ( "TriangleDown", "tile-triangle-down", _("Triangle _Down"), "<control>Down", _("Tile Triangle Down The Checked Windows"), inst.tile_triangle_down),
    ( "TriangleLeft", "tile-triangle-left", _("Triangle _Left"), "<control>Left", _("Tile Triangle Left The Checked Windows"), inst.tile_triangle_left),
    ( "TriangleRight", "tile-triangle-right", _("Triangle _Right"), "<control>Right", _("Tile Triangle Right The Checked Windows"), inst.tile_triangle_right),
    ( "Quad", "tile-quad", _("Tile _Quad"), "<control>4", _("Tile into 4 quadrants The Checked Windows"), inst.tile_quad),
    ( "Grid", "tile-grid", _("Tile _Grid"), "<control>G", _("Tile into an Arbitrary Grid The Checked Windows"), inst.dialog_grid),
    ( "Custom1Set", "tile-custom-1-set", _("Custom Tile 1 _Set"), "<alt>1", _("Edit Custom Tile 1 Settings"), inst.tile_custom_1_set),
    ( "Custom1Exe", "tile-custom-1-exe", _("Custom Tile _1 Run"), "<control>1", _("Execute Custom Tile 1"), inst.tile_custom_1_run),
    ( "Custom2Set", "tile-custom-2-set", _("Custom Tile 2 S_et"), "<alt>2", _("Edit Custom Tile 2 Settings"), inst.tile_custom_2_set),
    ( "Custom2Exe", "tile-custom-2-exe", _("Custom Tile _2 Run"), "<control>2", _("Execute Custom Tile 2"), inst.tile_custom_2_run),
    ( "UnTile", "gtk-undo", _("U_ndo Tiling"), "<control>Z", _("Undo the Latest Tiling Operation"), inst.undo_tiling),
    ( "InverTile", "invert-tiling", _("_Invert Tiling Order"), "<control>I", _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling),
    ( "CycleTile", "cycle-tiling", _("C_ycle Tiling Order"), "<control>Y", _("Cycle the Order of the Latest Tiling Operation"), inst.cycle_tiling),
    ( "Maximize", "maximize-checked-windows", _("_Maximize Windows"), "<control>M", _("Maximize The Checked Windows"), inst.maximize_checked_windows),
    ( "Unmaximize", "unmaximize-checked-windows", _("_Unmaximize Windows"), "<control>U", _("Unmaximize The Checked Windows"), inst.unmaximize_checked_windows),
    ( "Close", "close-checked-windows", _("_Close Windows"), "<control>C", _("Close The Checked Windows"), inst.close_checked_windows),
    ( "RowUp", "gtk-go-up", _("Move _Up"), "<alt>Up", _("Move the Selected Row Up"), inst.on_button_row_up_clicked),
    ( "RowDown", "gtk-go-down", _("Move _Down"), "<alt>Down", _("Move the Selected Row Down"), inst.on_button_row_down_clicked),
    ( "RowDelete", "gtk-remove", _("_Remove"), "Delete", _("Remove the Selected Row"), inst.on_button_row_delete_clicked),
    ( "ShowHideToolbar", "toolbar", _("Show/Hide _Toolbar"), None, _("Toggle Show/Hide Toolbar"), inst.show_hide_toolbar),
    ( "IncreaseToolbarIconsSize", "gtk-zoom-in", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
    ( "DecreaseToolbarIconsSize", "gtk-zoom-out", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
    ( "Help", "help-contents", _("_Help"), None, _("X Tile Project Home Page"), inst.on_help_menu_item_activated),
    ( "About", "gtk-about", _("_About"), None, _("About X Tile"), inst.dialog_about),
    ( "All_About", "gtk-about", _("_About"), None, _("About X Tile"), inst.dialog_about_all),
    ( "All_C", "close-checked-windows", _("_Close All"), None, _("Close All Windows"), inst.close_all),
    ( "All_U", "unmaximize-checked-windows", _("_Unmaximize All"), None, _("Unmaximize All Windows"), inst.unmaximize_all),
    ( "All_M", "maximize-checked-windows", _("_Maximize All"), None, _("Maximize All Windows"), inst.maximize_all),
    ( "All_4", "tile-quad", _("Tile All _Quad"), None, _("Tile All Windows Quad"), inst.tile_all_quad),
    ( "All_TD", "tile-triangle-down", _("Tile All Triangle _Down"), None, _("Tile All Windows Triangle Down"), inst.tile_all_triangle_down),
    ( "All_TU", "tile-triangle-up", _("Tile All Triangle _Up"), None, _("Tile All Windows Triangle Up"), inst.tile_all_triangle_up),
    ( "All_TR", "tile-triangle-right", _("Tile All Triangle _Right"), None, _("Tile All Windows Triangle Right"), inst.tile_all_triangle_right),
    ( "All_TL", "tile-triangle-left", _("Tile All Triangle _Left"), None, _("Tile All Windows Triangle Left"), inst.tile_all_triangle_left),
    ( "All_C2", "tile-custom-2-exe", _("Tile All Custom _2"), None, _("Tile All Windows Custom 2"), inst.tile_all_custom_2),
    ( "All_C1", "tile-custom-1-exe", _("Tile All Custom _1"), None, _("Tile All Windows Custom 1"), inst.tile_all_custom_1),
    ( "All_G", "tile-grid", _("Tile All _Grid"), None, _("Tile All Windows Grid"), inst.tile_all_grid),
    ( "All_H", "tile-horizontally", _("Tile All _Horizontally"), None, _("Tile All Windows Horizontally"), inst.tile_all_horizontally),
    ( "All_V", "tile-vertically", _("Tile All _Vertically"), None, _("Tile All Windows Vertically"), inst.tile_all_vertically),
    ( "All_Invert", "invert-tiling", _("_Invert Tiling Order"), None, _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling_all),
    ( "All_Undo", "gtk-undo", _("U_ndo Tiling"), None, _("Undo the Latest Tiling Operation"), inst.undo_tiling_all),
    ( "All_Cycle", "cycle-tiling", _("C_ycle Tiling Order"), None, _("Cycle the Order of the Latest Tiling Operation"), inst.cycle_tiling_all),
    ]

HELP_TEXT = """

NAME
   X Tile {} - Tile the Windows upon your Desktop

SYNOPSIS
   x-tile OPTION

DESCRIPTION
   This manual page briefly documents the command line arguments

OPTIONS

   w => open the x-tile main window without using the panel

   z => undo the latest tiling operation

   v => tile all opened windows vertically

   h => tile all opened windows horizontally

   u => tile all opened windows triangle-up

   d => tile all opened windows triangle-down

   l => tile all opened windows triangle-left

   r => tile all opened windows triangle-right

   q => quad tile all opened windows

   g = g 0 = g 0 0   => tile all opened windows in a grid with automatic rows and columns
   g rows = g rows 0 => tile all opened windows in a grid with given rows and automatic columns
   g 0 cols          => tile all opened windows in a grid with automatic rows and given columns
   g rows cols       => tile all opened windows in a grid with given rows and columns

   1 => custom tile 1 all opened windows

   2 => custom tile 2 all opened windows

   i => invert the order of the latest tiling operation

   y => cycle the order of the latest tiling operation

   m => maximize all opened windows

   M => unmaximize all opened windows

   c => close all opened windows

SEE ALSO
   https://www.giuspen.net/x-tile

AUTHORS
   Giuseppe Penone (aka giuspen) and Chris Camacho (aka Chris_C)

""".format(VERSION)



#
#      core.py
#
#      Copyright 2009-2022
#      Giuseppe Penone <giuspen@gmail.com>,
#      Chris Camacho (chris_c) <codifies@gmail.com>.
#
#      plus many thanks to  http://tronche.com/gui/x/xlib/
#                      and  http://tripie.sweb.cz/utils/wmctrl/
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#      MA 02110-1301, USA.

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
from gi.repository import GObject
import os
import sys
import ctypes
import webbrowser
import time
import subprocess
def module_exists(module_name):
    try:
        gi.require_version(module_name, '0.1')
    except:
        return False
    else:
        return True

if module_exists('AppIndicator3'):
    from gi.repository import AppIndicator3 as appindicator
    HAS_APPINDICATOR = True
elif module_exists('AyatanaAppIndicator3'):
    from gi.repository import AyatanaAppIndicator3 as appindicator
    HAS_APPINDICATOR = True
else:
    HAS_APPINDICATOR = False
import cons
import support
import tilings
ICONS_SIZE = {1: Gtk.IconSize.MENU, 2: Gtk.IconSize.SMALL_TOOLBAR, 3: Gtk.IconSize.LARGE_TOOLBAR,
              4: Gtk.IconSize.DND, 5: Gtk.IconSize.DIALOG}

class MockingGConf:
    """Minimal GConf replacement"""

    def _get_filepath(self, key):
        key_clean = key.replace("/", "-")
        return os.path.join(cons.CONFIG_DIR, key_clean)

    def add_dir(self, dirpath):
        if not os.path.isdir(cons.CONFIG_DIR):
            os.makedirs(cons.CONFIG_DIR)

    def get_string(self, key):
        ret_val = None
        if os.path.isfile(self._get_filepath(key)):
            with open(self._get_filepath(key), "r") as fd:
                ret_val = fd.read()
        return ret_val

    def set_string(self, key, value):
        with open(self._get_filepath(key), "w") as fd:
            fd.write(value)

    def get_int(self, key):
        the_string = self.get_string(key)
        return int(the_string) if the_string is not None else 0

    def set_int(self, key, value):
        self.set_string(key, str(value))


class InfoModel:
    """Holds the information"""

    def __init__(self):
        """Sets up and populates the Gtk.ListStore"""
        # 0:selected, 1:window ref, 2:title, 3:pixbuf of icon, 4:selected2, 5:cell_background
        self.gconf_client = MockingGConf()
        self.liststore = Gtk.ListStore(GObject.TYPE_BOOLEAN,
                                       GObject.TYPE_ULONG,
                                       GObject.TYPE_STRING,
                                       GObject.TYPE_PYOBJECT,
                                       GObject.TYPE_BOOLEAN,
                                       GObject.TYPE_STRING)
        self.process_picklist = set()
        self.process_blacklist = set()
        self.process_whitelist = set()

    def load_model(self, appletobj):
        """Populates the Gtk.ListStore"""
        self.liststore.clear()
        clients = []
        support.get_property(b"_NET_CLIENT_LIST", glob.root, glob.XA_WINDOW)
        for i in range(0, glob.num_items.value):
            clients.append(glob.ret_pointer[i])
        rows_num = 0
        support.get_property(b"_NET_CURRENT_DESKTOP", glob.root, glob.XA_CARDINAL)
        if bool(glob.ret_pointer) == False:
            curr_workspace_num = -2
            print("DEBUG warning _NET_CURRENT_DESKTOP improperly set")
        else: curr_workspace_num = glob.ret_pointer[0] # the number of the current workspace
        screen = Gdk.Screen.get_default()
        for client in clients:
            if support.is_window_sticky(client): continue
            support.get_property(b"_NET_WM_DESKTOP", client, glob.XA_CARDINAL)
            if bool(glob.ret_pointer): workspace_num = glob.ret_pointer[0]
            else:
                print("DEBUG warning _NET_WM_DESKTOP improperly set")
                workspace_num = curr_workspace_num
            if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == cons.STR_TRUE\
            and (workspace_num != curr_workspace_num or not support.is_window_in_curr_viewport(glob.desktop_width, glob.desktop_height, client)):
                continue
            if self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == cons.STR_TRUE\
            and support.is_window_hidden(client):
                continue
            support.get_property(b"_NET_WM_NAME", client, glob.str_atom)
            title=b"???"
            if bool(glob.ret_pointer)==False:
                print("DEBUG warning _NET_WM_NAME improperly set by application")
                support.get_property(b"WM_NAME", client, glob.str2_atom)
                if bool(glob.ret_pointer)==False:
                    print("DEBUG warning WM_NAME not set by application")
                else:
                    title = ctypes.string_at(glob.ret_pointer)
            else: title = ctypes.string_at(glob.ret_pointer)
            if title in cons.WINNAMES_BLACKLIST: continue
            pxb = support.get_icon(client)
            if pxb: pxb = pxb.scale_simple(24, 24, GdkPixbuf.InterpType.BILINEAR)
            support.get_property(b"_NET_WM_PID", client, glob.XA_CARDINAL)
            pid=0
            process_name=b"UNKNOWN"
            if bool(glob.ret_pointer)==False:
                print("DEBUG warning can't get PID _NET_WM_PID failed")
            else:
                pid = glob.ret_pointer[0]
                process_name = support.get_process_name(pid)
            # filter based on process name - NB beppie I'd like something better for this but dont know what!
            if len(process_name) > 6 and process_name[-6:] == "x-tile": continue
            if process_name in cons.PROCESSES_BLACKLIST: continue
            #print "win", pid, process_name
            #if pid > 0:
                #ppid = os.popen("ps -p %d -oppid=" % pid).read().strip()
                #pp_name = support.get_process_name(ppid)
                #print "pwin", ppid, pp_name
                #if pp_name in cons.PROCESSES_PARENT_BLACKLIST: continue
            if support.is_candidate_compiz_desktop(client): continue
            self.process_picklist.add(process_name)
            if process_name not in self.process_blacklist: # user filter
                if glob.num_monitors > 1:
                    win_geom = support.get_geom(client)
                    win_curr_monitor = screen.get_monitor_at_point(win_geom[0]+win_geom[2]/2,
                                                                   win_geom[1]+win_geom[3]/2)
                else: win_curr_monitor = 0
                if win_curr_monitor == 0: cell_background = None
                else: cell_background = 'gray'
                if process_name not in self.process_whitelist: flagged = False
                else: flagged = True
                if win_curr_monitor == 0: self.liststore.prepend([flagged, client, title.decode('utf-8'), pxb, False, cell_background])
                else: self.liststore.append([flagged, client, title.decode('utf-8'), pxb, False, cell_background])
                rows_num += 1
        if rows_num == 2:
            iter = self.liststore.get_iter_first()
            while iter != None:
                self.liststore[iter][0] = True
                iter = self.liststore.iter_next(iter)

    def flag_all_rows(self):
        """Flags All Rows"""
        if glob.num_monitors > 1:
            all_rows_flagged_on_monitor_1 = True
            iter = self.liststore.get_iter_first()
            while iter != None:
                if self.liststore[iter][0] == False:
                    all_rows_flagged_on_monitor_1 = False
                    break
                iter = self.liststore.iter_next(iter)
        else: all_rows_flagged_on_monitor_1 = False
        iter = self.liststore.get_iter_first()
        while iter != None:
            if not all_rows_flagged_on_monitor_1:
                self.liststore[iter][0] = True
                self.liststore[iter][4] = False
            else:
                self.liststore[iter][0] = False
                self.liststore[iter][4] = True
            iter = self.liststore.iter_next(iter)

    def unflag_all_rows(self):
        """Unflags All Rows"""
        iter = self.liststore.get_iter_first()
        while iter != None:
            self.liststore[iter][0] = False
            self.liststore[iter][4] = False
            iter = self.liststore.iter_next(iter)

    def get_checked_windows_list(self, undo_ready=False):
        """Returns the list of the checked windows"""
        checked_windows_list = [[],[]]
        if undo_ready:
            # undo_snap_vec is 0:win_id 1:is_maximized 2:x 3:y 4:width 5:height
            undo_snap_vec = []
        tree_iter = self.liststore.get_iter_first()
        while tree_iter != None:
            try: win_id = self.liststore[tree_iter][1]
            except: continue
            if self.liststore[tree_iter][0] == True:
                checked_windows_list[0].append(win_id)
            elif self.liststore[tree_iter][4] == True:
                checked_windows_list[1].append(win_id)
            if undo_ready and (self.liststore[tree_iter][0] or self.liststore[tree_iter][4]):
                if support.is_window_Vmax(win_id) or support.is_window_Hmax(win_id): is_maximized = 1
                else: is_maximized = 0
                win_geom = support.get_geom(win_id)
                undo_snap_vec.append([  str(win_id),
                                        str(is_maximized),
                                        str(win_geom[0]),
                                        str(win_geom[1]),
                                        str(win_geom[2]),
                                        str(win_geom[3])  ])
            tree_iter = self.liststore.iter_next(tree_iter)
        if undo_ready and undo_snap_vec:
            support.undo_snap_write(self.gconf_client, undo_snap_vec)
        return checked_windows_list

    def close_checked_windows(self):
        """Closes the checked windows and removes the rows from the model"""
        iter = self.liststore.get_iter_first()
        while iter != None:
            next_iter = self.liststore.iter_next(iter)
            if self.liststore[iter][0] == True or self.liststore[iter][4] == True:
                support.client_msg(self.liststore[iter][1], b"_NET_CLOSE_WINDOW", 0, 0, 0, 0, 0)
                glob.x11.XSync(glob.disp, False)
                self.liststore.remove(iter)
            iter = next_iter

    def row_up(self, iter):
        """Row up one position"""
        prev_iter = self.get_tree_iter_prev_sibling(iter)
        if prev_iter != None: self.liststore.swap(iter, prev_iter)

    def row_down(self, iter):
        """Row down one position"""
        subseq_iter = self.liststore.iter_next(iter)
        if subseq_iter != None: self.liststore.swap(iter, subseq_iter)

    def row_delete(self, iter):
        """Row remove"""
        self.liststore.remove(iter)

    def get_tree_iter_prev_sibling(self, node_iter):
        """Returns the previous sibling iter or None if the given iter is the first"""
        node_path = self.liststore.get_path(node_iter)
        sibling_index = len(node_path)-1
        prev_iter = None
        while prev_iter == None and node_path[sibling_index] > 0:
            node_path_list = list(node_path)
            node_path_list[sibling_index] -= 1
            prev_path = tuple(node_path_list)
            prev_iter = self.liststore.get_iter(prev_path)
        return prev_iter

    def get_model(self):
        """Returns the model"""
        return self.liststore


class GladeWidgetsWrapper:
    """Handles the retrieval of glade widgets"""

    def __init__(self, glade_file_path, gui_instance):
        try:
            self.glade_widgets = Gtk.Builder()
            self.glade_widgets.set_translation_domain(cons.APP_NAME)
            self.glade_widgets.add_from_file(glade_file_path)
            self.glade_widgets.connect_signals(gui_instance)
        except: print("Failed to load the glade file")

    def __getitem__(self, key):
        """Gives us the ability to do: wrapper['widget_name'].action()"""
        return self.glade_widgets.get_object(key)

    def __getattr__(self, attr):
        """Gives us the ability to do: wrapper.widget_name.action()"""
        new_widget = self.glade_widgets.get_object(attr)
        if new_widget is None: raise AttributeError('Widget %r not found' % attr)
        setattr(self, attr, new_widget)
        return new_widget


class XTile:
    """The application's main window"""

    def __init__(self, store):
        """Instantiate the Glade Widgets Wrapper, create the view,
        retrieves and stores the information about the running desktop self.geometry"""
        # create a variable pointing to the instance of the InfoModel class
        self.store = store
        # icons generation
        factory = Gtk.IconFactory()
        for filename in cons.ICONS_FILENAMES:
            stock_name = filename[:-4]
            filepath = os.path.join(cons.GLADE_PATH, filename)
            pixbuf = GdkPixbuf.Pixbuf.new_from_file(filepath)
            iconset = Gtk.IconSet(pixbuf)
            factory.add(stock_name, iconset)
        factory.add_default()
        # system settings
        try:
            gtk_settings = Gtk.Settings.get_default()
            gtk_settings.set_property("gtk-button-images", True)
            gtk_settings.set_property("gtk-menu-images", True)
        except: pass # older gtk do not have the property "gtk-menu-images"
        os.environ['UBUNTU_MENUPROXY'] = '0' # x-tile has custom stock icons not visible in appmenu
        self.cmd_line_only = False
        # instantiate the Glade Widgets Wrapper
        self.glade = GladeWidgetsWrapper(cons.GLADE_PATH + 'x-tile.glade', self)
        # ui manager
        actions = Gtk.ActionGroup("Actions")
        actions.add_actions(cons.get_entries(self))
        self.ui = Gtk.UIManager()
        self.ui.insert_action_group(actions, 0)
        self.glade.window.add_accel_group(self.ui.get_accel_group())
        self.ui.add_ui_from_string(cons.UI_INFO)
        # menubar add
        self.glade.vbox_main.pack_start(self.ui.get_widget("/MenuBar"), False, False, 0)
        self.glade.vbox_main.reorder_child(self.ui.get_widget("/MenuBar"), 0)
        # toolbar add
        self.glade.vbox_main.pack_start(self.ui.get_widget("/ToolBar"), False, False, 0)
        self.glade.vbox_main.reorder_child(self.ui.get_widget("/ToolBar"), 1)
        self.ui.get_widget("/ToolBar").set_style(Gtk.ToolbarStyle.ICONS)
        # create the view
        self.view = Gtk.TreeView(store.get_model())
        self.view.set_headers_visible(False)
        self.renderer_checkbox = Gtk.CellRendererToggle()
        self.renderer_checkbox.set_property('activatable', True)
        self.renderer_checkbox.connect('toggled', self.toggle_active, self.store.liststore)
        self.renderer_checkbox2 = Gtk.CellRendererToggle()
        self.renderer_checkbox2.set_property('activatable', True)
        self.renderer_checkbox2.connect('toggled', self.toggle_active2, self.store.liststore)
        self.renderer_pixbuf = Gtk.CellRendererPixbuf()
        self.renderer_text = Gtk.CellRendererText()
        self.columns = [None]*4
        self.columns[0] = Gtk.TreeViewColumn("Tile", self.renderer_checkbox, active=0) # active=0 <> read from column 0 of model
        self.columns[0].add_attribute(self.renderer_checkbox, "cell-background", 5)
        self.columns[1] = Gtk.TreeViewColumn("Tile", self.renderer_checkbox2, active=4) # active=4 <> read from column 4 of model
        self.columns[1].add_attribute(self.renderer_checkbox2, "cell-background", 5)
        self.columns[2] = Gtk.TreeViewColumn("Logo", self.renderer_pixbuf)
        self.columns[2].set_cell_data_func(self.renderer_pixbuf, self.make_pixbuf)
        self.columns[2].add_attribute(self.renderer_pixbuf, "cell-background", 5)
        self.columns[3] = Gtk.TreeViewColumn("Window Description", self.renderer_text, text=2) # text=2 <> read from column 2 of model
        self.columns[3].add_attribute(self.renderer_text, "cell-background", 5)
        for n in range(4): self.view.append_column(self.columns[n])
        if glob.num_monitors < 2: self.columns[1].set_visible(False)
        self.view.set_reorderable(True) # allow drag and drop reordering of rows
        self.view.set_tooltip_text(_("Use Drag and Drop to Sort the Rows"))
        self.view.connect('button-press-event', self.on_mouse_button_clicked_list)
        self.no_toggling_signals = False
        self.viewselection = self.view.get_selection()
        self.glade.scrolledwindow.add(self.view)
        self.glade.processadddialog.connect('key_press_event', self.on_key_press_processadddialog)
        self.glade.drawingarea.connect('draw', self.on_drawing_area_draw)
        self.glade.aboutdialog.set_version(cons.VERSION)
        self.glade.window.set_title(self.glade.window.get_title() + " " + cons.VERSION)
        self.gconf_client = MockingGConf()
        self.gconf_client.add_dir("/apps/x-tile/%s" % glob.screen_index)
        self.combobox_country_lang_init()

    def combobox_country_lang_init(self):
        """Init The Programming Languages Syntax Highlighting"""
        combobox = self.glade.combobox_country_language
        self.country_lang_liststore = Gtk.ListStore(str)
        combobox.set_model(self.country_lang_liststore)
        cell = Gtk.CellRendererText()
        combobox.pack_start(cell, True)
        combobox.add_attribute(cell, 'text', 0)
        for country_lang in cons.AVAILABLE_LANGS:
            self.country_lang_liststore.append([country_lang])
        combobox.set_active_iter(self.get_combobox_country_lang_iter(self.gconf_client.get_string(cons.GCONF_LANG)))
        self.glade.combobox_country_language.connect('changed', self.on_combobox_country_language_changed)

    def get_combobox_country_lang_iter(self, country_language):
        """Returns the Language iter Given the Language Name"""
        curr_iter = self.country_lang_liststore.get_iter_first()
        while curr_iter != None:
            if self.country_lang_liststore[curr_iter][0] == country_language: break
            else: curr_iter = self.country_lang_liststore.iter_next(curr_iter)
        else: return self.country_lang_liststore.get_iter_first()
        return curr_iter

    def on_combobox_country_language_changed(self, combobox):
        """New Country Language Choosed"""
        new_iter = self.glade.combobox_country_language.get_active_iter()
        new_lang = self.country_lang_liststore[new_iter][0]
        if new_lang != self.gconf_client.get_string(cons.GCONF_LANG):
            self.country_lang = new_lang
            support.dialog_info(_("The New Language will be Available Only After Restarting X Tile"), self.glade.window)
            self.gconf_client.set_string(cons.GCONF_LANG, new_lang)
    
    def status_icon_enable(self):
        """Creates the Stats Icon"""
        if self.use_appindicator():
            self.ind = appindicator.Indicator.new("x-tile", "indicator-messages", appindicator.IndicatorCategory.APPLICATION_STATUS)
            self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)
            self.ind.set_attention_icon("indicator-messages-new")
            for icp in ["/usr/share/icons/hicolor/scalable/apps/x-tile.svg", "/usr/local/share/icons/hicolor/scalable/apps/x-tile.svg", "glade/x-tile.svg"]:
                if os.path.isfile(icp):
                    icon_path = icp
                    break
            else: icon_path = cons.APP_NAME
            self.ind.set_icon(icon_path)
            self.ind.set_menu(self.ui.get_widget("/SysTrayMenu"))
        else:
            self.status_icon = Gtk.StatusIcon()
            self.status_icon.set_from_stock("tile-quad")
            self.status_icon.connect('button-press-event', self.on_mouse_button_clicked_systray)
            self.status_icon.set_tooltip_text(_("Tile the Windows Upon your X Desktop"))
    
    def on_mouse_button_clicked_systray(self, widget, event):
        """Catches mouse buttons clicks upon the system tray icon"""
        if event.button == 1: self.toggle_show_hide_main_window()
        elif event.button == 3: self.ui.get_widget("/SysTrayMenu").popup(None, None, None, None, event.button, event.time)
    
    def toggle_show_hide_main_window(self, *args):
        if self.win_on_screen: self.window_hide()
        else:
            self.window_position_restore()
            self.glade.window.show()
            self.reload_windows_list()
            self.win_on_screen = True
    
    def on_checkbutton_use_appind_toggled(self, checkbutton):
        """Use AppIndicator Toggled Handling"""
        if self.glade.checkbutton_systray_docking.get_active():
            former_active = True
            self.glade.checkbutton_systray_docking.set_active(False)
        else: former_active = False
        if checkbutton.get_active():
            self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_TRUE)
        else:
            self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_FALSE)
        if former_active:
            self.glade.checkbutton_systray_docking.set_active(True)
    
    def on_checkbutton_systray_docking_toggled(self, checkbutton):
        """SysTray Toggled Handling"""
        self.systray_on = checkbutton.get_active()
        if self.systray_on:
            if not self.use_appindicator():
                if "status_icon" in dir(self): self.status_icon.set_property('visible', True)
                else: self.status_icon_enable()
            else:
                if "ind" in dir(self): self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)
                else: self.status_icon_enable()
            self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', True)
            self.glade.checkbutton_start_minimized.set_sensitive(True)
            if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) != cons.STR_TRUE:
                self.gconf_client.set_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index, cons.STR_TRUE)
        else:
            if not self.use_appindicator():
                if not "status_icon" in dir(self): self.status_icon_enable()
                self.status_icon.set_property('visible', False)
            else:
                if not "ind" in dir(self): self.status_icon_enable()
                self.ind.set_status(appindicator.IndicatorStatus.PASSIVE)
            self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', False)
            self.glade.checkbutton_start_minimized.set_sensitive(False)
            if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) != cons.STR_FALSE:
                self.gconf_client.set_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index, cons.STR_FALSE)

    def on_checkbutton_start_minimized_toggled(self, checkbutton):
        """Start Minimized on SysTray Toggled Handling"""
        if checkbutton.get_active():
            if self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) != cons.STR_TRUE:
                self.gconf_client.set_string(cons.GCONF_SYSTRAY_START % glob.screen_index, cons.STR_TRUE)
        else:
            if self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) != cons.STR_FALSE:
                self.gconf_client.set_string(cons.GCONF_SYSTRAY_START % glob.screen_index, cons.STR_FALSE)
    
    def on_checkbutton_override_monitor_1_area_toggled(self, checkbutton):
        """Override Area 1 Checkbox was Toggled"""
        if checkbutton.get_active(): cons.OVERRIDE_1 = 1
        else:
            cons.OVERRIDE_1 = 0
            glob.read_monitors_areas()
            cons.X1 = glob.monitors_areas[0][0]
            cons.Y1 = glob.monitors_areas[0][1]
            cons.W1 = glob.monitors_areas[0][2]
            cons.H1 = glob.monitors_areas[0][3]
            self.glade.spinbutton_x1.set_value(cons.X1)
            self.glade.spinbutton_y1.set_value(cons.Y1)
            self.glade.spinbutton_w1.set_value(cons.W1)
            self.glade.spinbutton_h1.set_value(cons.H1)
        self.gconf_client.set_int(cons.GCONF_OVERRIDE_1 % glob.screen_index, cons.OVERRIDE_1)
        self.glade.vbox_override_area_1.set_sensitive(cons.OVERRIDE_1 == 1)

    def on_checkbutton_override_monitor_2_area_toggled(self, checkbutton):
        """Override Area 1 Checkbox was Toggled"""
        if checkbutton.get_active(): cons.OVERRIDE_2 = 1
        else:
            cons.OVERRIDE_2 = 0
            glob.read_monitors_areas()
            cons.X2 = glob.monitors_areas[1][0]
            cons.Y2 = glob.monitors_areas[1][1]
            cons.W2 = glob.monitors_areas[1][2]
            cons.H2 = glob.monitors_areas[1][3]
            self.glade.spinbutton_x2.set_value(cons.X2)
            self.glade.spinbutton_y2.set_value(cons.Y2)
            self.glade.spinbutton_w2.set_value(cons.W2)
            self.glade.spinbutton_h2.set_value(cons.H2)
        self.gconf_client.set_int(cons.GCONF_OVERRIDE_2 % glob.screen_index, cons.OVERRIDE_2)
        self.glade.vbox_override_area_2.set_sensitive(cons.OVERRIDE_2 == 1)

    def init_from_gconf(self):
        """Init the geometry and the spinbuttons"""
        glob.read_monitors_areas()
        self.glade.drawingarea.set_property("width-request", glob.drawing_rect[2])
        self.glade.drawingarea.set_property("height-request", glob.drawing_rect[3])
        self.custom_geoms_1 = []
        custom_geoms_str = self.gconf_client.get_string(cons.GCONF_CUSTOM_1 % glob.screen_index)
        if custom_geoms_str:
            custom_geoms_vec = custom_geoms_str.split(" ")
            for custom_geom in custom_geoms_vec:
                x, y, width, height = custom_geom.split(",")
                self.custom_geoms_1.append([int(x), int(y), int(width), int(height)])
        self.custom_geoms_2 = []
        custom_geoms_str = self.gconf_client.get_string(cons.GCONF_CUSTOM_2 % glob.screen_index)
        if custom_geoms_str:
            custom_geoms_vec = custom_geoms_str.split(" ")
            for custom_geom in custom_geoms_vec:
                x, y, width, height = custom_geom.split(",")
                self.custom_geoms_2.append([int(x), int(y), int(width), int(height)])
        if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == None:
            self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_TRUE)
        if self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == None:
            self.gconf_client.set_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index, cons.STR_FALSE)
        if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == None:
            self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, cons.STR_FALSE)
        # systray handling
        if not HAS_APPINDICATOR: self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_FALSE)
        else: self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_TRUE)
        if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == None:
            self.gconf_client.set_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index, cons.STR_FALSE)
        if self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) == None:
            self.gconf_client.set_string(cons.GCONF_SYSTRAY_START % glob.screen_index, cons.STR_FALSE)
        if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == cons.STR_TRUE:
            self.status_icon_enable()
            self.systray_on = True
            self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', True)
        else:
            self.systray_on = False
            self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', False)
        # exit after tile
        self.glade.exit_after_tile_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE)
        # monitor 1 handling
        if self.gconf_client.get_int(cons.GCONF_OVERRIDE_1 % glob.screen_index) == 1:
            cons.OVERRIDE_1 = 1
            cons.X1 = self.gconf_client.get_int(cons.GCONF_X1 % glob.screen_index)
            cons.Y1 = self.gconf_client.get_int(cons.GCONF_Y1 % glob.screen_index)
            cons.W1 = self.gconf_client.get_int(cons.GCONF_W1 % glob.screen_index)
            cons.H1 = self.gconf_client.get_int(cons.GCONF_H1 % glob.screen_index)
            glob.monitors_areas[0] = [cons.X1, cons.Y1, cons.W1, cons.H1]
        else:
            cons.X1 = glob.monitors_areas[0][0]
            cons.Y1 = glob.monitors_areas[0][1]
            cons.W1 = glob.monitors_areas[0][2]
            cons.H1 = glob.monitors_areas[0][3]
        # monitor 2 handling
        if glob.num_monitors > 1:
            if self.gconf_client.get_int(cons.GCONF_OVERRIDE_2 % glob.screen_index) == 1:
                cons.OVERRIDE_2 = 1
                cons.X2 = self.gconf_client.get_int(cons.GCONF_X2 % glob.screen_index)
                cons.Y2 = self.gconf_client.get_int(cons.GCONF_Y2 % glob.screen_index)
                cons.W2 = self.gconf_client.get_int(cons.GCONF_W2 % glob.screen_index)
                cons.H2 = self.gconf_client.get_int(cons.GCONF_H2 % glob.screen_index)
                glob.monitors_areas[1] = [cons.X2, cons.Y2, cons.W2, cons.H2]
            else:
                cons.X2 = glob.monitors_areas[1][0]
                cons.Y2 = glob.monitors_areas[1][1]
                cons.W2 = glob.monitors_areas[1][2]
                cons.H2 = glob.monitors_areas[1][3]
        # grid parameters
        grid_rows = self.gconf_client.get_int(cons.GCONF_GRID_ROWS % glob.screen_index)
        if grid_rows: cons.GRID_ROWS = grid_rows
        grid_cols = self.gconf_client.get_int(cons.GCONF_GRID_COLS % glob.screen_index)
        if grid_cols: cons.GRID_COLS = grid_cols

        key = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
        if key not in ICONS_SIZE: self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, 3)
        self.ui.get_widget("/ToolBar").set_property("icon-size",
              ICONS_SIZE[self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)])

        blacklist = self.gconf_client.get_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index)
        if blacklist not in [None, ""]:
            for element in blacklist.split():
                self.store.process_blacklist.add(element)
        whitelist = self.gconf_client.get_string(cons.GCONF_PROCESS_WHITELIST % glob.screen_index)
        if whitelist not in [None, ""]:
            for element in whitelist.split():
                self.store.process_whitelist.add(element)

        self.glade.checkbutton_override_monitor_1_area.set_active(cons.OVERRIDE_1 == 1)
        self.glade.checkbutton_override_monitor_2_area.set_active(cons.OVERRIDE_2 == 1)
        self.glade.spinbutton_x1.set_value(cons.X1)
        self.glade.spinbutton_y1.set_value(cons.Y1)
        self.glade.spinbutton_w1.set_value(cons.W1)
        self.glade.spinbutton_h1.set_value(cons.H1)
        self.glade.spinbutton_x2.set_value(cons.X2)
        self.glade.spinbutton_y2.set_value(cons.Y2)
        self.glade.spinbutton_w2.set_value(cons.W2)
        self.glade.spinbutton_h2.set_value(cons.H2)
        self.glade.vbox_override_area_1.set_sensitive(cons.OVERRIDE_1 == 1)
        self.glade.vbox_override_area_2.set_sensitive(cons.OVERRIDE_2 == 1)
        self.glade.checkbutton_override_monitor_2_area.set_sensitive(glob.num_monitors > 1)
        self.win_size_n_pos = {}
        self.win_size_n_pos['win_size'] = [self.gconf_client.get_int(cons.GCONF_WIN_SIZE_W % glob.screen_index),
                                           self.gconf_client.get_int(cons.GCONF_WIN_SIZE_H % glob.screen_index)]
        if 0 not in self.win_size_n_pos['win_size']:
            self.glade.window.resize(self.win_size_n_pos['win_size'][0], self.win_size_n_pos['win_size'][1])

    def window_position_restore(self):
        """Restore window size and position"""
        self.win_size_n_pos['win_position'] = [self.gconf_client.get_int(cons.GCONF_WIN_POSITION_X % glob.screen_index),
                                               self.gconf_client.get_int(cons.GCONF_WIN_POSITION_Y % glob.screen_index)]
        self.glade.window.move(self.win_size_n_pos['win_position'][0], self.win_size_n_pos['win_position'][1])

    def toolbar_icons_size_increase(self, *args):
        """Increase the Size of the Toolbar Icons"""
        toolbar_icon_size = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
        if toolbar_icon_size == 5:
            support.dialog_info(_("The Size of the Toolbar Icons is already at the Maximum Value"), self.glade.window)
            return
        toolbar_icon_size += 1
        self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, toolbar_icon_size)
        self.ui.get_widget("/ToolBar").set_property("icon-size", ICONS_SIZE[toolbar_icon_size])

    def toolbar_icons_size_decrease(self, *args):
        """Decrease the Size of the Toolbar Icons"""
        toolbar_icon_size = self.gconf_client.get_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index)
        if toolbar_icon_size == 1:
            support.dialog_info(_("The Size of the Toolbar Icons is already at the Minimum Value"), self.glade.window)
            return
        toolbar_icon_size -= 1
        self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, toolbar_icon_size)
        self.ui.get_widget("/ToolBar").set_property("icon-size", ICONS_SIZE[toolbar_icon_size])

    def toggle_active(self, cell, path, model):
        """Toggles the Active state"""
        model[path][0] = not model[path][0]
        model[path][4] = False

    def toggle_active2(self, cell, path, model):
        """Toggles the Active state"""
        model[path][4] = not model[path][4]
        model[path][0] = False

    def toggle_exit_after_tile(self, *args):
        """Toggles the flag Exit After Tile"""
        if not self.no_toggling_signals:
            if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE:
                self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_FALSE)
            else: self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_TRUE)

    def toggle_do_not_list_minimized(self, *args):
        """Toggles the flag Do Not List Minimized Windows in List"""
        if not self.no_toggling_signals:
            if self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == cons.STR_TRUE:
                self.gconf_client.set_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index, cons.STR_FALSE)
            else: self.gconf_client.set_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index, cons.STR_TRUE)
            self.store.load_model(self)

    def toggle_only_curr_workspace(self, *args):
        """Toggles the flag Only Current Workspace Windows in List"""
        if not self.no_toggling_signals:
            if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == cons.STR_TRUE:
                self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, cons.STR_FALSE)
            else: self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, cons.STR_TRUE)
            self.store.load_model(self)

    def is_window_visible(self):
        """Returns True if the window is visible, False otherwise"""
        return self.glade.window.get_property("visible")

    def flag_all_rows(self, *args):
        """Flags All Rows"""
        self.store.flag_all_rows()

    def unflag_all_rows(self, *args):
        """Unflags All Rows"""
        self.store.unflag_all_rows()

    def make_pixbuf(self, treeviewcolumn, cell, tree_model, tree_iter, data):
        """Function to associate the pixbuf to the cell renderer"""
        try:
            pixbuf = tree_model[tree_iter][3]
            cell.set_property('pixbuf', pixbuf)
        except: pass

    def on_window_delete_event(self, widget, event, data=None):
        """Before close the application: no checks needed"""
        self.quit_application()
        return True # do not propogate the delete event

    def on_configwindow_delete_event(self, widget, event, data=None):
        """Destroy the config window"""
        self.on_configwin_close_button_clicked()
        return True # do not propogate the delete event

    def dialog_preferences(self, *args):
        """Open the Config Window"""
        if self.glade.configwindow.get_property("visible") == True: return
        self.no_toggling_signals = True
        self.glade.current_workspace_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == cons.STR_TRUE)
        self.glade.do_not_list_minimized_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == cons.STR_TRUE)
        self.glade.checkbutton_systray_docking.set_active(self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == cons.STR_TRUE)
        self.glade.checkbutton_start_minimized.set_active(self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) == cons.STR_TRUE)
        self.glade.checkbutton_use_appind.set_active(self.gconf_client.get_string(cons.GCONF_USE_APPIND % glob.screen_index) == cons.STR_TRUE)
        if not HAS_APPINDICATOR:
            self.glade.checkbutton_use_appind.set_sensitive(False)
        self.glade.show_toolbar_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index) == cons.STR_TRUE)
        self.no_toggling_signals = False
        self.glade.configwindow.show_all()

    def on_configwin_close_button_clicked(self, *args):
        """Hide the Dialog"""
        self.glade.configwindow.hide()
        cons.X1 = int(self.glade.spinbutton_x1.get_value())
        cons.Y1 = int(self.glade.spinbutton_y1.get_value())
        cons.W1 = int(self.glade.spinbutton_w1.get_value())
        cons.H1 = int(self.glade.spinbutton_h1.get_value())
        if cons.OVERRIDE_1 == 1:
            glob.monitors_areas[0] = [cons.X1, cons.Y1, cons.W1, cons.H1]
            self.gconf_client.set_int(cons.GCONF_X1 % glob.screen_index, cons.X1)
            self.gconf_client.set_int(cons.GCONF_Y1 % glob.screen_index, cons.Y1)
            self.gconf_client.set_int(cons.GCONF_W1 % glob.screen_index, cons.W1)
            self.gconf_client.set_int(cons.GCONF_H1 % glob.screen_index, cons.H1)
        if glob.num_monitors > 1:
            cons.X2 = int(self.glade.spinbutton_x2.get_value())
            cons.Y2 = int(self.glade.spinbutton_y2.get_value())
            cons.W2 = int(self.glade.spinbutton_w2.get_value())
            cons.H2 = int(self.glade.spinbutton_h2.get_value())
            if cons.OVERRIDE_2 == 1:
                glob.monitors_areas[1] = [cons.X2, cons.Y2, cons.W2, cons.H2]
                self.gconf_client.set_int(cons.GCONF_X2 % glob.screen_index, cons.X2)
                self.gconf_client.set_int(cons.GCONF_Y2 % glob.screen_index, cons.Y2)
                self.gconf_client.set_int(cons.GCONF_W2 % glob.screen_index, cons.W2)
                self.gconf_client.set_int(cons.GCONF_H2 % glob.screen_index, cons.H2)

    def on_button_row_up_clicked(self, *args):
        """Move the selected row up of one position"""
        model, iter = self.viewselection.get_selected()
        if iter == None:
            support.dialog_warning(_("No Row is Selected"), self.glade.window)
            return
        self.store.row_up(iter)

    def on_button_row_down_clicked(self, *args):
        """Move the selected row up of one position"""
        model, iter = self.viewselection.get_selected()
        if iter == None:
            support.dialog_warning(_("No Row is Selected"), self.glade.window)
            return
        self.store.row_down(iter)

    def on_button_row_delete_clicked(self, *args):
        """Remove the selected row"""
        model, iter = self.viewselection.get_selected()
        if iter == None:
            support.dialog_warning(_("No Row is Selected"), self.glade.window)
            return
        self.store.row_delete(iter)

    def on_button_add_filter_clicked(self, *args):
        """Application Filter Add"""
        self.process_add_list_exist_or_create()
        self.process_add_liststore.clear()
        for element in self.store.process_picklist:
            self.process_add_liststore.append([element])
        response = self.glade.processadddialog.run()
        self.glade.processadddialog.hide()
        if response != 1: return
        model, iter = self.process_add_treeviewselection.get_selected()
        if iter == None: return
        self.store.process_blacklist.add(model[iter][0])
        self.filter_list_update()

    def on_button_remove_filter_clicked(self, *args):
        """Application Filter Remove"""
        model, iter = self.filter_treeviewselection.get_selected()
        if iter == None:
            support.dialog_warning(_("No Application Selected!"), self.glade.window)
            return
        self.store.process_blacklist.remove(model[iter][0])
        self.filter_list_update()

    def dialog_filter(self, action):
        """Application's Filter Dialog"""
        self.filter_list_exist_or_create()
        self.filter_list_update()
        self.glade.filterdialog.run()
        self.glade.filterdialog.hide()
        if len(self.store.process_blacklist) > 0:
            self.gconf_client.set_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index, " ".join(self.store.process_blacklist))
        else: self.gconf_client.set_string(cons.GCONF_PROCESS_BLACKLIST % glob.screen_index, "")
        self.reload_windows_list()

    def on_button_add_white_clicked(self, *args):
        """Application Whitelist Add"""
        self.process_add_list_exist_or_create()
        self.process_add_liststore.clear()
        for element in self.store.process_picklist:
            self.process_add_liststore.append([element])
        response = self.glade.processadddialog.run()
        self.glade.processadddialog.hide()
        if response != 1: return
        model, iter = self.process_add_treeviewselection.get_selected()
        if iter == None: return
        self.store.process_whitelist.add(model[iter][0])
        self.white_list_update()

    def on_button_remove_white_clicked(self, *args):
        """Application Whitelist Remove"""
        model, iter = self.white_treeviewselection.get_selected()
        if iter == None:
            support.dialog_warning(_("No Application Selected!"), self.glade.window)
            return
        self.store.process_whitelist.remove(model[iter][0])
        self.white_list_update()

    def dialog_selected_by_default(self, action):
        """Dialog to select a list of windows to be flagged by Default"""
        self.white_list_exist_or_create()
        self.white_list_update()
        self.glade.whitedialog.run()
        self.glade.whitedialog.hide()
        if len(self.store.process_whitelist) > 0:
            self.gconf_client.set_string(cons.GCONF_PROCESS_WHITELIST % glob.screen_index, " ".join(self.store.process_whitelist))
        else: self.gconf_client.set_string(cons.GCONF_PROCESS_WHITELIST % glob.screen_index, "")
        self.reload_windows_list()

    def filter_list_update(self):
        """Updates the Process Filter List"""
        self.filter_liststore.clear()
        for element in self.store.process_blacklist:
            self.filter_liststore.append([element])

    def white_list_update(self):
        """Updates the Process White List"""
        self.white_liststore.clear()
        for element in self.store.process_whitelist:
            self.white_liststore.append([element])

    def filter_list_exist_or_create(self):
        """If The List Was Never Used, this will Create It"""
        if not "filter_liststore" in dir(self):
            self.filter_liststore = Gtk.ListStore(str)
            self.filter_treeview = Gtk.TreeView(self.filter_liststore)
            self.filter_treeview.set_headers_visible(False)
            self.filter_renderer_text = Gtk.CellRendererText()
            self.filter_column = Gtk.TreeViewColumn("Application", self.filter_renderer_text, text=0)
            self.filter_treeview.append_column(self.filter_column)
            self.filter_treeviewselection = self.filter_treeview.get_selection()
            self.glade.scrolledwindow_filter.add(self.filter_treeview)
            self.glade.scrolledwindow_filter.show_all()

    def white_list_exist_or_create(self):
        """If The List Was Never Used, this will Create It"""
        if not "white_liststore" in dir(self):
            self.white_liststore = Gtk.ListStore(str)
            self.white_treeview = Gtk.TreeView(self.white_liststore)
            self.white_treeview.set_headers_visible(False)
            self.white_renderer_text = Gtk.CellRendererText()
            self.white_column = Gtk.TreeViewColumn("Application", self.white_renderer_text, text=0)
            self.white_treeview.append_column(self.white_column)
            self.white_treeviewselection = self.white_treeview.get_selection()
            self.glade.scrolledwindow_white.add(self.white_treeview)
            self.glade.scrolledwindow_white.show_all()

    def process_add_list_exist_or_create(self):
        """If The List Was Never Used, this will Create It"""
        if not "process_add_liststore" in dir(self):
            self.process_add_liststore = Gtk.ListStore(str)
            self.process_add_treeview = Gtk.TreeView(self.process_add_liststore)
            self.process_add_treeview.set_headers_visible(False)
            self.process_add_renderer_text = Gtk.CellRendererText()
            self.process_add_column = Gtk.TreeViewColumn("Application", self.process_add_renderer_text, text=0)
            self.process_add_treeview.append_column(self.process_add_column)
            self.process_add_treeviewselection = self.process_add_treeview.get_selection()
            self.process_add_treeview.connect('button-press-event', self.on_mouse_button_clicked_process_add)
            self.glade.scrolledwindow_process_add.add(self.process_add_treeview)
            self.glade.scrolledwindow_process_add.show_all()

    def on_mouse_button_clicked_process_add(self, widget, event):
        """Catches mouse buttons clicks"""
        if event.button != 1: return
        if event.type == Gdk.EventType._2BUTTON_PRESS: self.glade.processadddialog_button_ok.clicked()

    def on_key_press_processadddialog(self, widget, event):
        """Catches AnchorHandle Dialog key presses"""
        keyname = Gdk.keyval_name(event.keyval)
        if keyname == "Return": self.glade.processadddialog_button_ok.clicked()

    def dialog_grid(self, *args):
        """Open the Grid Dialog"""
        dialog = Gtk.Dialog(title=_("Grid Details"),
                                    parent=self.glade.window,
                                    flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
                                    buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                                    Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT) )
        dialog.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
        content_area = dialog.get_content_area()
        hbox_rows = Gtk.HBox()
        label_rows = Gtk.Label(label=_("Rows"))
        spinbutton_rows = Gtk.SpinButton()
        adj_rows = spinbutton_rows.get_adjustment()
        #adj_rows.set_all(cons.GRID_ROWS, 1, 100, 1, 0, 0)
        adj_rows = Gtk.Adjustment(value=1,lower=1,upper=100, step_increment=1, page_increment=0)
        spinbutton_rows.set_adjustment(adj_rows)
        hbox_rows.pack_start(label_rows, True, True, 0)
        hbox_rows.pack_start(spinbutton_rows, True, True, 0)
        hbox_cols = Gtk.HBox()
        label_cols = Gtk.Label(label=_("Columns"))
        spinbutton_cols = Gtk.SpinButton()
        #adj_cols = spinbutton_cols.get_adjustment()
        #adj_cols.set_all(cons.GRID_COLS, 1, 100, 1, 0, 0)
        adj_cols = Gtk.Adjustment(value=1, lower=1,upper=100, step_increment=1, page_increment=0)
        spinbutton_cols.set_adjustment(adj_cols)
        hbox_cols.pack_start(label_cols, True, True, 0)
        hbox_cols.pack_start(spinbutton_cols, True, True, 0)
        content_area.pack_start(hbox_rows, True, True, 0)
        content_area.pack_start(hbox_cols, True, True, 0)
        def on_key_press_enter_password_dialog(widget, event):
            if Gdk.keyval_name(event.keyval) == "Return":
                button_box = dialog.get_action_area()
                buttons = button_box.get_children()
                buttons[0].clicked() # first is the ok button
        dialog.connect("key_press_event", on_key_press_enter_password_dialog)
        dialog.show_all()
        response = dialog.run()
        cons.GRID_ROWS = int(spinbutton_rows.get_value())
        cons.GRID_COLS = int(spinbutton_cols.get_value())
        dialog.destroy()
        if response != Gtk.ResponseType.ACCEPT: return
        self.gconf_client.set_int(cons.GCONF_GRID_ROWS % glob.screen_index, cons.GRID_ROWS)
        self.gconf_client.set_int(cons.GCONF_GRID_COLS % glob.screen_index, cons.GRID_COLS)
        self.tile_grid()

    def use_appindicator(self):
        """Will we use AppIndicator?"""
        return self.gconf_client.get_string(cons.GCONF_USE_APPIND % glob.screen_index) == cons.STR_TRUE

    def quit_application_totally(self, *args):
        """The process is Shut Down"""
        if not self.use_appindicator() and "status_icon" in dir(self):
            self.status_icon.set_visible(False)
        self.save_win_pos_n_size()
        self.glade.window.destroy()
        Gtk.main_quit()
    
    def save_win_pos_n_size(self):
        """Destroy the window"""
        actual_win_size = list(self.glade.window.get_size())
        actual_win_pos = list(self.glade.window.get_position())
        if actual_win_size != self.win_size_n_pos['win_size']:
            self.win_size_n_pos['win_size'] = actual_win_size
            self.gconf_client.set_int(cons.GCONF_WIN_SIZE_W % glob.screen_index, self.win_size_n_pos['win_size'][0])
            self.gconf_client.set_int(cons.GCONF_WIN_SIZE_H % glob.screen_index, self.win_size_n_pos['win_size'][1])
        if actual_win_pos != self.win_size_n_pos['win_position']:
            self.win_size_n_pos['win_position'] = actual_win_pos
            self.gconf_client.set_int(cons.GCONF_WIN_POSITION_X % glob.screen_index, self.win_size_n_pos['win_position'][0])
            self.gconf_client.set_int(cons.GCONF_WIN_POSITION_Y % glob.screen_index, self.win_size_n_pos['win_position'][1])
    
    def window_hide(self):
        """Hide the Window"""
        self.save_win_pos_n_size()
        self.glade.window.hide()
        self.win_on_screen = False
    
    def quit_application(self, *args):
        """Hide the window"""
        if not self.systray_on: self.quit_application_totally()
        else: self.window_hide()

    def launch_application(self):
        """Show the main window and all child widgets"""
        self.init_from_gconf()
        self.window_position_restore()
        self.glade.window.show_all()
        if glob.is_compiz_running:
            self.glade.checkbutton_dest_workspace.set_active(False)
            self.glade.checkbutton_dest_workspace.hide()
            self.glade.spinbutton_dest_workspace.hide()
        show_toolbar = self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index)
        if show_toolbar == None: self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, cons.STR_TRUE)
        elif show_toolbar == cons.STR_FALSE: self.ui.get_widget("/ToolBar").hide()
        self.win_on_screen = True
        if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == cons.STR_FALSE:
            self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', False)
        elif self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) == cons.STR_TRUE:
            self.glade.window.hide()
            self.win_on_screen = False

    def reload_windows_list(self, *args):
        """Reloads the Windows List"""
        self.store.load_model(self)

    def close_checked_windows(self, *args):
        """Closes the Checked Windows"""
        number_of_windows = len(self.store.get_checked_windows_list())
        if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
        else: self.store.close_checked_windows()

    def maximize_checked_windows(self, *args):
        """Maximizes the Checked Windows"""
        checked_windows_lists = self.store.get_checked_windows_list(True)
        checked_windows_list = checked_windows_lists[0] + checked_windows_lists[1]
        number_of_windows = len(checked_windows_list)
        if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
        else:
            for checked_window in checked_windows_list: support.maximize(checked_window)
            self.check_exit_after_tile()

    def unmaximize_checked_windows(self, *args):
        """Unmaximizes the Checked Windows"""
        checked_windows_lists = self.store.get_checked_windows_list(True)
        checked_windows_list = checked_windows_lists[0] + checked_windows_lists[1]
        number_of_windows = len(checked_windows_list)
        if number_of_windows < 1: support.dialog_warning(_('No Windows Checked'), self.glade.window)
        else:
            for checked_window in checked_windows_list: support.unmaximize(checked_window)
            self.check_exit_after_tile()

    def tile_vertically(self, *args):
        """Tile the Checked Windows Vertically"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "v")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_vertically(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_horizontally(self, *args):
        """Tile the Checked Windows Horizontally"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "h")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_horizontally(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_quad(self, *args):
        """Tile the Checked Windows Quad"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "q")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_quad(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_grid(self):
        """Tile the Checked Windows in a rows by cols grid"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "g")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_grid(cons.GRID_ROWS,
                          cons.GRID_COLS,
                          checked_windows_list,
                          glob.monitors_areas,
                          self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_custom_1_set(self, *args):
        """The User Wants to Set/Update the Custom Tiling"""
        self.last_custom = 1
        self.glade.customsetdialog.set_title(_("Edit Custom Tile Settings") + " 1")
        response = self.glade.customsetdialog.run()
        self.glade.customsetdialog.hide()
        if response != 1: return
        custom_geoms_str = ""
        for win_geom in self.custom_geoms_1:
            custom_geoms_str += " %s,%s,%s,%s" % (win_geom[0], win_geom[1], win_geom[2], win_geom[3])
        if custom_geoms_str != "": custom_geoms_str = custom_geoms_str[1:]
        self.gconf_client.set_string(cons.GCONF_CUSTOM_1 % glob.screen_index, custom_geoms_str)

    def tile_custom_2_set(self, *args):
        """The User Wants to Set/Update the Custom Tiling"""
        self.last_custom = 2
        self.glade.customsetdialog.set_title(_("Edit Custom Tile Settings") + " 2")
        response = self.glade.customsetdialog.run()
        self.glade.customsetdialog.hide()
        if response != 1: return
        custom_geoms_str = ""
        for win_geom in self.custom_geoms_2:
            custom_geoms_str += " %s,%s,%s,%s" % (win_geom[0], win_geom[1], win_geom[2], win_geom[3])
        if custom_geoms_str != "": custom_geoms_str = custom_geoms_str[1:]
        self.gconf_client.set_string(cons.GCONF_CUSTOM_2 % glob.screen_index, custom_geoms_str)

    def on_drawing_area_draw(self, drawing_area, cairo_context):
        """Drawing Area was Exposed"""
        if self.last_custom == 1:
            if self.custom_geoms_1: self.custom_geoms_draw(self.custom_geoms_1, cairo_context)
        elif self.last_custom == 2:
            if self.custom_geoms_2: self.custom_geoms_draw(self.custom_geoms_2, cairo_context)

    def on_mouse_button_clicked_list(self, widget, event):
        """Catches mouse buttons clicks"""
        if event.button == 3:
            self.ui.get_widget("/ListMenu").popup(None, None, None, None, event.button, event.time)

    def on_button_update_custom_tiling_clicked(self, button):
        """Let's Get Positions and Size of the Flagged Windows"""
        checked_windows_list = self.store.get_checked_windows_list()
        if self.last_custom == 1:
            self.custom_geoms_1 = []
            for win_id in checked_windows_list[0] + checked_windows_list[1]:
                win_geom = support.get_geom(win_id)
                self.custom_geoms_1.append([win_geom[0], win_geom[1], win_geom[2], win_geom[3]])
        elif self.last_custom == 2:
            self.custom_geoms_2 = []
            for win_id in checked_windows_list[0] + checked_windows_list[1]:
                win_geom = support.get_geom(win_id)
                self.custom_geoms_2.append([win_geom[0], win_geom[1], win_geom[2], win_geom[3]])
        self.glade.drawingarea.queue_draw()

    def custom_geoms_draw(self, custom_geoms, cairo_context):
        """Draw Custom Geometries"""
        rgb_idx = 0
        for i, win_geom in enumerate(custom_geoms):
            cairo_context.set_source_rgb(*cons.DRAW_RGBS[rgb_idx])
            cairo_context.rectangle(win_geom[0]/cons.DRAW_SCALE, win_geom[1]/cons.DRAW_SCALE,
                         win_geom[2]/cons.DRAW_SCALE, win_geom[3]/cons.DRAW_SCALE)
            cairo_context.fill()
            cairo_context.set_source_rgb(0, 0, 0)
            cairo_context.set_font_size(13)
            cairo_context.move_to(win_geom[0]/cons.DRAW_SCALE + win_geom[2]/(2*cons.DRAW_SCALE),
                       win_geom[1]/cons.DRAW_SCALE + win_geom[3]/(2*cons.DRAW_SCALE))
            cairo_context.show_text(str(i+1))
            if rgb_idx + 1 < len(cons.DRAW_RGBS): rgb_idx += 1
            else: rgb_idx = 0

    def get_dest_ws(self):
        """Get Destination Desktop"""
        if self.glade.checkbutton_dest_workspace.get_active():
            return int(self.glade.spinbutton_dest_workspace.get_value()-1)
        return -1

    def invert_tiling(self, *args):
        """Invert the order of the latest tiling operation"""
        # get the win_id and win_geom of the latest tiled windows
        latest_tiling_geoms = []
        undo_snap_str = self.gconf_client.get_string(cons.GCONF_UNDO % glob.screen_index)
        if not undo_snap_str: return
        undo_snap_vec = undo_snap_str.split(" ")
        doubleundo_snap_vec = []
        for element in undo_snap_vec:
            win_id, is_maximized, x, y, width, height = element.split(",")
            win_geom = support.get_geom(int(win_id))
            latest_tiling_geoms.append({'win_id': win_id, 'win_geom': win_geom})
        #print "latest_tiling_geoms", latest_tiling_geoms
        # generate the win_id and win_geom of the next tiled windows
        next_tiling_geoms = []
        for i, element in enumerate(latest_tiling_geoms):
            next_tiling_geoms.append({'win_id': element['win_id'],
                                      'win_geom': latest_tiling_geoms[-1-i]['win_geom']})
        #print "next_tiling_geoms", next_tiling_geoms
        # tile the windows
        for element in next_tiling_geoms:
            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(int(element['win_id'])))
            support.moveresize(int(element['win_id']),
                               int(element['win_geom'][0]),
                               int(element['win_geom'][1]),
                               int(element['win_geom'][2]),
                               int(element['win_geom'][3]),
                               self.get_dest_ws())
        if doubleundo_snap_vec:
            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
        self.check_exit_after_tile()

    def cycle_tiling(self, *args):
        """Cycle the order of the latest tiling operation"""
        # get the win_id and win_geom of the latest tiled windows
        latest_tiling_geoms = []
        undo_snap_str = self.gconf_client.get_string(cons.GCONF_UNDO % glob.screen_index)
        if not undo_snap_str: return
        undo_snap_vec = undo_snap_str.split(" ")
        doubleundo_snap_vec = []
        for element in undo_snap_vec:
            win_id, is_maximized, x, y, width, height = element.split(",")
            win_geom = support.get_geom(int(win_id))
            latest_tiling_geoms.append({'win_id': win_id, 'win_geom': win_geom})
        #print "latest_tiling_geoms", latest_tiling_geoms
        # generate the win_id and win_geom of the next tiled windows
        next_tiling_geoms = []
        for i, element in enumerate(latest_tiling_geoms):
            next_tiling_geoms.append({'win_id': element['win_id'],
                                      'win_geom': latest_tiling_geoms[(i+1)%len(latest_tiling_geoms)]['win_geom']})
        #print "next_tiling_geoms", next_tiling_geoms
        # tile the windows
        for element in next_tiling_geoms:
            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(int(element['win_id'])))
            support.moveresize(int(element['win_id']),
                               int(element['win_geom'][0]),
                               int(element['win_geom'][1]),
                               int(element['win_geom'][2]),
                               int(element['win_geom'][3]),
                               self.get_dest_ws())
        if doubleundo_snap_vec:
            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
        self.check_exit_after_tile()

    def undo_tiling(self, *args):
        """Undo the Latest Tiling Operation"""
        undo_snap_str = self.gconf_client.get_string(cons.GCONF_UNDO % glob.screen_index)
        if not undo_snap_str: return
        undo_snap_vec = undo_snap_str.split(" ")
        doubleundo_snap_vec = []
        for element in undo_snap_vec:
            win_id, is_maximized, x, y, width, height = element.split(",")
            # save current state for eventual undo of the undo
            win_id_int = int(win_id)
            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(win_id_int))
            # proceed with the undo
            if int(is_maximized) == 1: support.maximize(win_id_int)
            else: support.moveresize(win_id_int, int(x), int(y), int(width), int(height), self.get_dest_ws())
        if doubleundo_snap_vec:
            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
        self.check_exit_after_tile()

    def tile_custom_1_run(self, *args):
        """Tile N Windows According to the User Defined Tiling"""
        custom_geoms_str = self.gconf_client.get_string(cons.GCONF_CUSTOM_1 % glob.screen_index)
        if not custom_geoms_str:
            support.dialog_info(_("The Custom Tile 1 was Not Set Yet: Click the Menu Item 'Tile->Custom Tile 1 Set'"), self.glade.window)
            return
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "1")
        custom_geoms_vec = custom_geoms_str.split(" ")
        windows_list = self.store.get_checked_windows_list(True)
        windows_list = windows_list[0] + windows_list[1]
        win_num = 0
        for checked_window in windows_list:
            x, y, width, height = custom_geoms_vec[win_num].split(",")
            support.moveresize(checked_window, int(x), int(y), int(width), int(height), self.get_dest_ws())
            if win_num + 1 < len(custom_geoms_vec): win_num += 1
            else: break
        self.check_exit_after_tile()

    def tile_custom_2_run(self, *args):
        """Tile N Windows According to the User Defined Tiling"""
        custom_geoms_str = self.gconf_client.get_string(cons.GCONF_CUSTOM_2 % glob.screen_index)
        if not custom_geoms_str:
            support.dialog_info(_("The Custom Tile 2 was Not Set Yet: Click the Menu Item 'Tile->Custom Tile 2 Set'"), self.glade.window)
            return
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "2")
        custom_geoms_vec = custom_geoms_str.split(" ")
        windows_list = self.store.get_checked_windows_list(True)
        windows_list = windows_list[0] + windows_list[1]
        win_num = 0
        for checked_window in windows_list:
            x, y, width, height = custom_geoms_vec[win_num].split(",")
            support.moveresize(checked_window, int(x), int(y), int(width), int(height), self.get_dest_ws())
            if win_num + 1 < len(custom_geoms_vec): win_num += 1
            else: break
        self.check_exit_after_tile()

    def tile_triangle_up(self, *args):
        """Tile 3 Windows in Triangle Up Scheme"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "u")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_triangle_up(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_triangle_down(self, *args):
        """Tile 3 Windows in Triangle Down Scheme"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "d")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_triangle_down(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_triangle_left(self, *args):
        """Tile 3 Windows in Triangle Left Scheme"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "l")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_triangle_left(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def tile_triangle_right(self, *args):
        """Tile 3 Windows in Triangle Right Scheme"""
        self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "r")
        checked_windows_list = self.store.get_checked_windows_list(True)
        tilings.tile_triangle_right(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
        self.check_exit_after_tile()

    def show_hide_toolbar(self, menuitem, data=None):
        """Show/Hide the Toolbar"""
        if not self.no_toggling_signals:
            if self.ui.get_widget("/ToolBar").get_property('visible') == True:
                self.ui.get_widget("/ToolBar").hide()
                self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, cons.STR_FALSE)
            else:
                self.ui.get_widget("/ToolBar").show()
                self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, cons.STR_TRUE)
    
    def check_exit_after_tile(self):
        """Check if the Exit After Tile is Active and Eventually Quit"""
        if self.cmd_line_only: return
        if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE:
            self.quit_application()
    
    def dialog_about(self, menuitem, data=None):
        """Show the About Dialog and hide it when a button is pressed"""
        self.glade.aboutdialog.run()
        self.glade.aboutdialog.hide()
    
    def on_help_menu_item_activated(self, menuitem, data=None):
        """Show the application's Instructions"""
        webbrowser.open("https://www.giuspen.net/x-tile/")
    
    def hide_and_process(self, command_str):
        """Hide the X Tile Window if Visible, then Process the Command"""
        if self.win_on_screen: self.window_hide()
        subprocess.call(command_str, shell=True)
    
    def undo_tiling_all(self, *args):
        """Just Undo the Latest Tiling Operation"""
        self.hide_and_process("x-tile z &")
    
    def invert_tiling_all(self, *args):
        """Invert the Order of the Latest Tiling Operation"""
        self.hide_and_process("x-tile i &")
    
    def cycle_tiling_all(self, *args):
        """Cycle the Order of the Latest Tiling Operation"""
        self.hide_and_process("x-tile y &")
    
    def tile_all_vertically(self, *args):
        """Just tile Vertically all opened windows"""
        self.hide_and_process("x-tile v &")
    
    def tile_all_horizontally(self, *args):
        """Just tile Horizontally all opened windows"""
        self.hide_and_process("x-tile h &")
    
    def tile_all_triangle_up(self, *args):
        """Just tile Triangle Up all opened windows"""
        self.hide_and_process("x-tile u &")
    
    def tile_all_triangle_down(self, *args):
        """Just tile Triangle Down all opened windows"""
        self.hide_and_process("x-tile d &")
    
    def tile_all_triangle_left(self, *args):
        """Just tile Triangle Left all opened windows"""
        self.hide_and_process("x-tile l &")
    
    def tile_all_triangle_right(self, *args):
        """Just tile Triangle Right all opened windows"""
        self.hide_and_process("x-tile r &")
    
    def tile_all_quad(self, *args):
        """Just tile Quad all opened windows"""
        self.hide_and_process("x-tile q &")
    
    def tile_all_grid(self, *args):
        """Just tile Grid all opened windows"""
        self.hide_and_process("x-tile g &")
    
    def tile_all_custom_1(self, *args):
        """Just tile Custom 1 all opened windows"""
        self.hide_and_process("x-tile 1 &")
    
    def tile_all_custom_2(self, *args):
        """Just tile Custom 2 all opened windows"""
        self.hide_and_process("x-tile 2 &")
    
    def maximize_all(self, *args):
        """Maximize all opened windows"""
        self.hide_and_process("x-tile m &")
    
    def unmaximize_all(self, *args):
        """Unmaximize all opened windows"""
        self.hide_and_process("x-tile M &")
    
    def close_all(self, *args):
        """Close all opened windows"""
        self.hide_and_process("x-tile c &")
    
    def dialog_about_all(self, *args):
        """Show the About Dialog and hide it when a button is pressed"""
        glade = GladeWidgetsWrapper(cons.GLADE_PATH + 'x-tile.glade', self)
        glade.aboutdialog.set_version(cons.VERSION)
        glade.aboutdialog.run()
        glade.aboutdialog.destroy()
        del glade
